/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "HomgLine2D.hh"

using namespace BIAS;
using namespace std;

bool
HomgLine2D::GetIntersectionsWithImage(unsigned int width, unsigned int height,
				      unsigned int coo[4])
{
  int intersectioncount = 0;
  HomgPoint2D edges[4]; // 0=left, 1=right, 2=top, 3=bottom
  HomgLine2D left(-1.0, 0.0, 0.0);
  HomgLine2D right(-1.0, 0.0, width-1);
  HomgLine2D top(0.0, -1.0, 0.0);
  HomgLine2D bottom(0.0, -1.0, height-1);

  edges[0] = Intersection(left);
  edges[1] = Intersection(right);
  edges[2] = Intersection(top);
  edges[3] = Intersection(bottom);

//    cerr << "HomghLine2D.cc:" << endl;
//    for (int i =0; i<4; i++){
//      cerr << "edges[" << i << "] : " << edges[i] << endl;
//    }

  // deal with compiler precision
  for (unsigned int i=0; i<4; i++){
    if (edges[i][0] < 0.0 && edges[i][0] > -HOMGLINE2D_EPS)
      edges[i][0] = 0.0;
    if (edges[i][1] < 0.0 && edges[i][1] > -HOMGLINE2D_EPS)
      edges[i][1] = 0.0;
    if (edges[i][0] > width &&  
	edges[i][0] < width +HOMGLINE2D_EPS)
      edges[i][0] = width;
    if (edges[i][1] > height &&  
	edges[i][1] < height +HOMGLINE2D_EPS)
      edges[i][1] = height;
  }

  // left
  if (((edges[0][2] <-HOMGLINE2D_EPS) || (edges[0][2] > HOMGLINE2D_EPS) )&&
      (edges[0][1]>=0 && edges[0][1]<=height-1)){
    intersectioncount++;
    coo[0] = 0;
    coo[1] = (unsigned int)rint(edges[0][1]);
  } 
  // right
  if (((edges[1][2] <-HOMGLINE2D_EPS) || (edges[0][2] > HOMGLINE2D_EPS) )&&
      (edges[1][1]>=0 && edges[1][1]<=height-1)){
    if (intersectioncount==0){
      coo[0] = width - 1;
      coo[1] = (unsigned int)rint(edges[1][1]/edges[1][2]);
    } else {
      coo[2] = width - 1;
      coo[3] = (unsigned int)rint(edges[1][1]/edges[1][2]);
    }
    intersectioncount++;
  } 
  // top
  if (((edges[2][2] <-HOMGLINE2D_EPS) || (edges[2][2] > HOMGLINE2D_EPS) )&&
      (edges[2][0]>=0 && edges[2][0]<=width-1)){
    switch (intersectioncount){
    case 0:
      coo[0] = (unsigned int)rint(edges[2][0]/edges[2][2]);
      coo[1] = 0; 
      break;
    case 1:
      coo[2] = (unsigned int)rint(edges[2][0]/edges[2][2]);
      coo[3] = 0; 
      break;
    default:
      break;
    }
    intersectioncount++;
  } 
  // bottom
  if (((edges[3][2] <-HOMGLINE2D_EPS) || (edges[3][2] > HOMGLINE2D_EPS) )&&
      (edges[3][0]>=0 && edges[3][0]<=width-1)){
    switch (intersectioncount){
    case 0:
      coo[0] = (unsigned int)rint(edges[3][0]/edges[3][2]);
      coo[1] = height -1; 
      break;
    case 1:
      coo[2] = (unsigned int)rint(edges[3][0]/edges[3][2]);
      coo[3] = height -1; 
      break;
    case 2:
      if (sqrt(pow((float)(coo[0]-coo[2]),2) + pow((float)(coo[1]-coo[3]),2))< 
	  sqrt(pow((float)(coo[0]-(unsigned int)rint(edges[3][0]/edges[3][2])),2) + 
	       pow((float)(coo[1]-(height -1)),2))){
	coo[2] = (unsigned int)rint(edges[3][0]/edges[3][2]);
	coo[3] = height -1;
      }  
      break;
    default:
      break;
    }
    intersectioncount++;
  } 


  return (intersectioncount >1);
}







