/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Quaternion.hh"

using namespace BIAS;
using namespace std;

template<class QUAT_TYPE> 
Quaternion<QUAT_TYPE>::Quaternion(QUAT_TYPE i, QUAT_TYPE j,
                                  QUAT_TYPE k, QUAT_TYPE r) :  
  Vector4<QUAT_TYPE>(i, j, k, r)
{
}

template<class QUAT_TYPE> 
int Quaternion<QUAT_TYPE>::GetAxisAngle(Vector3<QUAT_TYPE>& axis, 
                                        QUAT_TYPE& angle) const
{
  // nicked from OpenSG
  int res=0;
  Vector3<QUAT_TYPE> q((*this)[0], (*this)[1], (*this)[2]);
  QUAT_TYPE len = QUAT_TYPE(q.NormL2());
  
  if(len > QUATERNION_EPSILON) {
    q /= len;
    axis[0]  = q[0];
    axis[1]  = q[1];
    axis[2]  = q[2];
    angle = QUAT_TYPE(2.0 * acos((*this)[3]));
  } else {
    axis[0] = 0.0;
    axis[1] = 0.0;
    axis[2] = 1.0;
    angle = 0.0;
  }
  return res;
}

template<class QUAT_TYPE> 
int Quaternion<QUAT_TYPE>::GetRotationMatrix(BIAS::RMatrixBase& R) const
{
  // nicked from OpenSG
  R[0][0] = 1.0 - 2.0 * ((*this)[1] * (*this)[1] +
                         (*this)[2] * (*this)[2]);
  R[0][1] =       2.0 * ((*this)[0] * (*this)[1] +
                         (*this)[2] * (*this)[3]);
  R[0][2] =       2.0 * ((*this)[2] * (*this)[0] -
                         (*this)[1] * (*this)[3]);

  R[1][0] =       2.0 * ((*this)[0] * (*this)[1] -
                         (*this)[2] * (*this)[3]);
  R[1][1] = 1.0 - 2.0 * ((*this)[2] * (*this)[2] +
                         (*this)[0] * (*this)[0]);
  R[1][2] =       2.0 * ((*this)[1] * (*this)[2] +
                         (*this)[0] * (*this)[3]);

  R[2][0] =       2.0 * ((*this)[2] * (*this)[0] +
                         (*this)[1] * (*this)[3]);
  R[2][1] =       2.0 * ((*this)[1] * (*this)[2] -
                         (*this)[0] * (*this)[3]);
  R[2][2] = 1.0 - 2.0 * ((*this)[1] * (*this)[1] +
                         (*this)[0] * (*this)[0]);
  return 0;
}
