/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "bias_config.h"

#include "Image.cpp"
#include "ImageConvert.cpp"
#include "Operators.cpp"
#include "ImageDraw.cpp"
#include "ImageConvertForeign.cpp"

using namespace BIAS;
using namespace std;

#define INSTANCE(type)\
template class Image<type>;\
template class ImageDraw<type>;\
template int ImageConvert::IP_ToGrey(Image<type> &img);\
template int ImageConvert::ToGrey(const Image<type>& source, Image<type>& dest);\
template int ImageConvert::ToRGB(Image<type>& source, Image<type>& dest);\
template int ImageConvert::ToHSV(Image<type>& source,Image<type>& dest);\
template int ImageConvert::ToHSL(Image<type>& source,Image<type>& dest);\
template int ImageConvert::TohsL(Image<type>& source,Image<type>& dest);\
template int ImageConvert::ToInterleaved(Image<type>& source,\
                                         Image<type>& dest);\
template int ImageConvert::ToInterleavedRGB(Image<type>& red, \
                                            Image<type>& green,\
                                            Image<type>& blue,\
                                            Image<type>& dest);\
template int ImageConvert::ToPlanar(Image<type>& source,\
                                    Image<type>& dest);\
template int ImageConvert::FromInterleaved(Image<type>& source,\
                                    Image<type>& Image1, Image<type>& Image2,\
                                    Image<type>& Image3);\
template int ImageConvert::IP_RGBToGrey_(Image<type> &img);\
template int ImageConvert::IP_YUV422ToGrey_(Image<type> &img);\
template int ImageConvert::BGRToRGB_(Image<type>& source, Image<type>& dest);\
template int ImageConvert::YUV422ToRGB_(Image<type>& source,\
                                        Image<type>& dest);\
template int ImageConvert::GreyToRGB_(Image<type>& source,\
                                      Image<type>& dest);\
template int ImageConvert::RGBToGrey_(const Image<type>& source,\
                                      Image<type>& dest);\
template int ImageConvert::HSLToGrey_(const Image<type>& source,\
                                      Image<type>& dest);\
template int ImageConvert::YUV422ToGrey_(const Image<type> &source,\
                                         Image<type>& dest);\
template int ImageConvert::ToPlanarYUYV422_(Image<type>& source,\
                                            Image<type>& dest);    \
template int ImageConvert::ToPlanarRGB_(Image<type>& source,\
                                        Image<type>& dest);\
template int ImageConvert::ToInterleavedYUYV422_(Image<type>& source,\
                                                 Image<type>& dest);\
template int ImageConvert::ToInterleavedRGB_(Image<type>& source,\
                                             Image<type>& dest);\
template int ImageConvert::FromInterleavedYUYV422_(Image<type>& source,\
                                            Image<type>& Y, \
                                            Image<type>& U,\
                                            Image<type>& V);\
template int ImageConvert::FromInterleavedRGB_(Image<type>& source,\
                                        Image<type>& R, \
                                        Image<type>& G,\
                                        Image<type>& B);\
template int ImageConvert::BayerToGrey_(const Image<type>& source,\
                                       Image<type>& dest);\
template int ImageConvert::BayerToRGB_(Image<type>& source,\
                                       Image<type>& dest);\
template int ImageConvert::BayerToRGBSlow_(Image<type>& source,\
                                       Image<type>& dest);

#ifdef BIAS_HAVE_DV
#define INSTDV(dvtype)\
template int ImageConvert::DVToRGB_(const Image<dvtype>& source,\
                                    Image<dvtype>& dest,unsigned char quality);
#else
#define INSTDV(dvtype)
#endif


#ifdef BIAS_HAVE_OPENCV
#define INSTOPENCV(cvt)\
template int ImageConvert::BIAS2ipl(const Image<cvt>& source, IplImage* &dest);\
template int ImageConvert::Ipl2BIAS(IplImage* source, Image<cvt>& dest);
#else
#define INSTOPENCV(cvt)
#endif // HAVE_OPENCV

#ifdef HAVE_VXL
#define INSTVXL(type3)\
template int ImageConvert::BIAS2vil(Image<type3>& source, vil_image **dest);\
template int ImageConvert::vil2BIAS(vil_image *source, Image<type3>& dest);
#else
#define INSTVXL(type3)
#endif // HAVE_VXL

#define INSTI(atype)   INSTANCE(atype) INSTOPENCV(atype) INSTDV(atype)

INSTI(unsigned char)
// INSTVXL(unsigned char)
INSTI(float)
INSTVXL(float)


#ifdef BUILD_IMAGE_INT
INSTI(int)
INSTVXL(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INSTI(char)
INSTVXL(char)
//INSTI(char)
  //INSTVXL(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INSTI(short)
INSTVXL(short)
#endif
#ifdef BUILD_IMAGE_USHORT
INSTI(unsigned short)
INSTVXL(unsigned short)
#endif
#ifdef BUILD_IMAGE_DOUBLE
INSTI(double)
INSTVXL(double)
#endif
#ifdef BUILD_IMAGE_UINT
INSTI(unsigned int)
INSTVXL(unsigned int)
#endif

