/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// HACK
#ifdef WIN32
#  pragma warning (disable: 4100)
#endif

#include "Tensor3D3x3x3.hh"
#include "Matrix3x3.hh"

using namespace BIAS;
using namespace std;

template <class T>
Tensor3D3x3x3<T>::Tensor3D3x3x3()
  : Tensor3D<T>(3, 3, 3, (T)0.0)
{}

template <class T>
Tensor3D3x3x3<T>::~Tensor3D3x3x3()
{
  this->Destroy();
}

template <class T>
void Tensor3D3x3x3<T>::NewSize(const int i, const int j, const int k, 
                               const T& deflt)
{
  BIASERR("do not call NewSize for Tensor3D3x3x3<T>, it has fixed size of 3x3x3"); 
  //BIASASSERT(false);
  exit(-1);
}

template <class T>
void Tensor3D3x3x3<T>::newsize(const int i, const int j, const int k, 
                               const T& deflt)
{
  BIASERR("do not call newsize for Tensor3D3x3x3<T>, it has fixed size of 3x3x3"); 
  //BIASASSERT(false);
  exit(-1);
}


template <class T>
void Tensor3D3x3x3<T>::postmultiply1(const Matrix3x3<T> &M,
                                     Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p=0; p<3; p++)
            v += this->_DataArray[p][j][k] * M[p][i];
          result(i,j,k) = (T)v; // we cast to the type to avoid warnings
        }
}


template <class T>
void Tensor3D3x3x3<T>::postmultiply2(const BIAS::Matrix3x3<T> &M,
                                     Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p = 0; p < 3; ++p)
            v += this->_DataArray[i][p][k] * M[p][j];
          result(i,j,k) = (T)v;
        }
}


template <class T>
void Tensor3D3x3x3<T>::postmultiply3(const BIAS::Matrix3x3<T> &M,
                                     Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p = 0; p < 3; ++p)
            v += this->_DataArray[i][j][p] * M[p][k];
          result(i,j,k) = (T)v;
        }
}


template <class T>
void Tensor3D3x3x3<T>::premultiply1(const BIAS::Matrix3x3<T> &M,
                                    Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p = 0; p < 3; ++p)
            v +=  M[i][p] * this->_DataArray[p][j][k];
          result(i,j,k) = (T)v;
        }
}


template <class T>
void Tensor3D3x3x3<T>::premultiply2(const BIAS::Matrix3x3<T> &M,
                                    Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p = 0; p < 3; ++p)
            v +=  M[j][p] * this->_DataArray[i][p][k];
          result(i,j,k) = (T)v;
        }
}


template <class T>
void Tensor3D3x3x3<T>::premultiply3(const BIAS::Matrix3x3<T> &M,
                                    Tensor3D3x3x3<T> &result) const
{
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      for (int k=0; k<3; k++)
        {
          double v = 0;
          for (int p = 0; p < 3; ++p)
            v +=  M[k][p] * this->_DataArray[i][j][p];
          result(i,j,k) = (T)v;
        }
}

