## 
## Try to find OpenEXR, 
## an high dynamic raneg imaeg library.
## (see  www.openexr.org)
## 
## Once run this will define: 
## 
## OPENEXR_FOUND
## OPENEXR_INCLUDE_DIR
## OPENEXR_LIBRARIES
## OPENEXR_LINK_DIRECTORIES
##
## Jan Woetzel 07/2004
## www.mip.informatik.uni-kiel.de/~jw
##

## tested with OpenEXR 1.2.1 on: 
## -Linux (Suse, gcc 9.2)
## -MS Windows XP, Visual Studio .NET 2003 7.1

##
## set (additional) directories to seacrh for headers: 
##
SET(OPENEXR_POSSIBLE_INC_DIRS
  ${OPENEXR_DIR}/include
  ${OPENEXR_HOME}/include
  $ENV{OPENEXR_HOME}/include
  
  ${OPENEXR_DIR}/include/OpenEXR
  ${OPENEXR_HOME}/include/OpenEXR
  $ENV{OPENEXR_HOME}/include/OpenEXR
  
  /usr/include/OpenEXR
  /usr/local/include/OpenEXR
)
#MESSAGE("DBG OPENEXR_POSSIBLE_INC_DIRS=${OPENEXR_POSSIBLE_INC_DIRS}")

##
## find specific headers fo rinclude dir(s):
##
FIND_PATH(OPENEXR_HALF_INCLUDE_DIR half.h
  ${OPENEXR_POSSIBLE_INC_DIRS}
  )
#MESSAGE("DBG OPENEXR_HALF_INCLUDE_DIR=${OPENEXR_HALF_INCLUDE_DIR}")


##
## find (all) required libraries: 
##

## set additional possible dirs to search for libs: 
SET(OPENEXR_POSSIBLE_LIB_DIRS
  ${OPENEXR_DIR}/lib
  ${OPENEXR_DIR}
  ${OPENEXR_HOME}/lib
  ${OPENEXR_HOME}
  $ENV{OPENEXR_HOME}/lib
  $ENV{OPENEXR_HOME}

  ## libs may reside in subdirs on Windows or source tree on Linux
  $ENV{OPENEXR_HOME}/lib/half
  $ENV{OPENEXR_HOME}/Half
  ${OPENEXR_DIR}/lib/half
  ${OPENEXR_DIR}/Half/
  
  $ENV{OPENEXR_HOME}/lib/iex
  $ENV{OPENEXR_HOME}/Iex
  ${OPENEXR_DIR}/lib/iex
  ${OPENEXR_DIR}/Iex

  $ENV{OPENEXR_HOME}/lib/ilmimf
  $ENV{OPENEXR_HOME}/ilmimf
  ${OPENEXR_DIR}/lib/ilmimf
  ${OPENEXR_DIR}/ilmimf

  $ENV{OPENEXR_HOME}/lib/imath
  $ENV{OPENEXR_HOME}/imath
  ${OPENEXR_DIR}/lib/imath
  ${OPENEXR_DIR}/imath

  /usr/lib
  /usr/local/lib
  )

FIND_LIBRARY(OPENEXR_Half_LIBRARY
  NAMES Half Halfd
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_Half_LIBRARY=${OPENEXR_Half_LIBRARY}")


FIND_LIBRARY(OPENEXR_IlmImf_LIBRARY
  NAMES IlmImf IlmImfd
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_IlmImf_LIBRARY=${OPENEXR_IlmImf_LIBRARY}")


FIND_LIBRARY(OPENEXR_Imath_LIBRARY
  NAMES Imath Imathd
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_Imath_LIBRARY=${OPENEXR_Imath_LIBRARY}")


FIND_LIBRARY(OPENEXR_Iex_LIBRARY
  NAMES Iex Iexd
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})  
#MESSAGE("DBG OPENEXR_Iex_LIBRARY=${OPENEXR_Iex_LIBRARY}")


FIND_LIBRARY(OPENEXR_z_LIBRARY
  NAMES z zlib zlibd
  PATHS
  /lib
  /usr/lib
  /usr/local/lib
  ${OPENEXR_HOME}/lib
  $ENV{OPENEXR_HOME}/lib
  ${OPENEXR_HOME}/lib/zlib
  $ENV{OPENEXR_HOME}/lib/zlib
  )
#MESSAGE("DBG OPENEXR_z_LIBRARY=${OPENEXR_z_LIBRARY}")


##
## evaluate results: 
##
IF(OPENEXR_Half_LIBRARY)
  IF(OPENEXR_IlmImf_LIBRARY)
    IF(OPENEXR_Imath_LIBRARY)
      IF(OPENEXR_Iex_LIBRARY)
        IF(OPENEXR_z_LIBRARY)
          IF (OPENEXR_HALF_INCLUDE_DIR)
            # OK.
	    SET(OPENEXR_FOUND TRUE)
	    SET(OPENEXR_INCLUDE_DIR ${OPENEXR_HALF_INCLUDE_DIR})
	    SET(OPENEXR_LIBRARIES 
	      ${OPENEXR_Half_LIBRARY} 
	      ${OPENEXR_IlmImf_LIBRARY} 
	      ${OPENEXR_Imath_LIBRARY} 
	      ${OPENEXR_Iex_LIBRARY} 
	      ${OPENEXR_z_LIBRARY} )
	    # HACK: use just one of the potentially multiple paths: 
	    GET_FILENAME_COMPONENT(OPENEXR_LINK_DIRECTORIES ${OPENEXR_IlmImf_LIBRARY} PATH)

	  #ELSE (OPENEXR_HALF_INCLUDE_DIR)
	  #  MESSAGE(SEND_ERROR "OPENEXR_HALF_INCLUDE_DIR not set")
	  ENDIF (OPENEXR_HALF_INCLUDE_DIR)
	  
	#ELSE(OPENEXR_z_LIBRARY)
	#  MESSAGE(SEND_ERROR "OPENEXR_z_LIBRARY not set")
	ENDIF(OPENEXR_z_LIBRARY)
	
      #ELSE(OPENEXR_Iex_LIBRARY)
      # MESSAGE(SEND_ERROR "OPENEXR_Iex_LIBRARY not set")
      ENDIF(OPENEXR_Iex_LIBRARY)
      
    #ELSE(OPENEXR_Imath_LIBRARY)
    #  MESSAGE(SEND_ERROR "OPENEXR_Imath_LIBRARY not set")
    ENDIF(OPENEXR_Imath_LIBRARY)
    
  #ELSE(OPENEXR_IlmImf_LIBRARY)
  #  MESSAGE(SEND_ERROR "OPENEXR_IlmImf_LIBRARY not set")
  ENDIF(OPENEXR_IlmImf_LIBRARY)  
  
#ELSE(OPENEXR_Half_LIBRARY)
#  MESSAGE(SEND_ERRROR "OPENEXR_Half_LIBRARY not set")
ENDIF(OPENEXR_Half_LIBRARY)



MARK_AS_ADVANCED(
  OPENEXR_HALF_INCLUDE_DIR
  OPENEXR_Half_LIBRARY
  OPENEXR_IlmImf_LIBRARY
  OPENEXR_Imath_LIBRARY
  OPENEXR_Iex_LIBRARY
  OPENEXR_z_LIBRARY
  )

