## 
## This module finds OpenSG and determines where the
## include files and libraries are. 
## On Unix/Linux it relies on the output of osg-config.
## This code sets the following variables:
## 
##
##
## OPENSG_FOUND       = system has OPENSG lib
##
## OPENSG_LIBRARIES   = full path to the libraries
##    on Unix/Linux with additional linker flags from "osg-config --libs"
## 
## CMAKE_OPENSG_CXX_FLAGS  = Unix compiler flags for OPENSG, essentially "`osg-config --cxxflags`"
##
## OPENSG_INCLUDE_DIR      = where to find headers 
##
## OPENSG_LINK_DIRECTORIES = link directories, useful for rpath on Unix
## OPENSG_EXE_LINKER_FLAGS = rpath on Unix
##
## Evers, Jan Woetzel 2004,2005
##

IF(WIN32)
  #MESSAGE(SEND_ERROR "FindOpenSG.cmake: OpenSG not (yet) supported on WIN32. Please complete the find script.")  
  # HACK for now (JW)

SET(OSGROOT   "$ENV{OSGROOT}")

IF("${OSGROOT}" STREQUAL "") 
  SET(OPENSG_HOME "C:/Program Files/OpenSG")
ELSE("${OSGROOT}" STREQUAL "") 
 STRING(REGEX REPLACE "[\\]" "/" OPENSG_HOME "${OSGROOT}")
#	SET(OPENSG_HOME "${OSGROOT}")
#MESSAGE(${OPENSG_HOME})
ENDIF("${OSGROOT}" STREQUAL "") 

  SET(OPENSG_DEFINITIONS "/D_OSG_HAVE_CONFIGURED_H_ /DOSG_BUILD_DLL /DOSG_DEBUG /DOSG_WITH_GLUT /DOSG_WITH_GIF /DOSG_WITH_TIF /DOSG_WITH_JPG ")
  SET(OPENSG_INCLUDE_DIR 
    "${OPENSG_HOME}/include"
    "C:/lib/include"
  )
  SET(OPENSG_LIBRARIES
    # MSVCPRTD.lib MSVCRTD.lib winmm.lib wsock32.lib 
    debug OSGBaseD optimized OSGBase
    debug OSGSystemD optimized OSGSystem 
    debug OSGWindowGLUTD optimized OSGWindowGLUT 
    debug OSGWindowWIN32D optimized OSGWindowWIN32 
    glut32
    glu32 
    opengl32
#   stlport_vc71
#   tif32 libjpeg
  )  
  SET(OPENSG_LINK_DIRECTORIES
   "${OPENSG_HOME}/lib"
#   "${OPENSG_HOME}/lib/glut_3.7/lib"
#   "${OPENSG_HOME}/lib/jpg6b/lib"
#   "${OPENSG_HOME}/lib/tiff-3.5.4/lib"
#  "C:/lib/lib"
  )

  #MESSAGE(SEND_ERROR "${OPENSG_LINK_DIRECTORIES}")
  ## global HACK
  ADD_DEFINITIONS    (${OPENSG_DEFINITIONS})
  INCLUDE_DIRECTORIES(${OPENSG_INCLUDE_DIR})
  LINK_DIRECTORIES   (${OPENSG_LINK_DIRECTORIES})
  LINK_LIBRARIES     (${OPENSG_LIBRARIES})
  SET(OPENSG_FOUND TRUE)


ELSE(WIN32)  ## --------------------------------------------------------------

  IF(UNIX) 
    SET(OPENSG_CONFIG_PREFER_PATH "$ENV{OPENSG_HOME}/bin" CACHE STRING "preferred path to OpenSG (osg-config)")
    FIND_PROGRAM(OPENSG_CONFIG osg-config
      ${OPENSG_CONFIG_PREFER_PATH}
      /usr/bin/
      )
    # MESSAGE("DBG OPENSG_CONFIG ${OPENSG_CONFIG}")
    
    IF (OPENSG_CONFIG) 
      # OK, found osg-config. 
      SET(OSG_MODULES "Base System GLUT")

      # determine if we want optimized OpenSG using "--opt"
      SET(OPENSG_OPT FALSE)
      IF (CMAKE_BUILD_TYPE MATCHES "Release")
	SET(OPENSG_OPT TRUE)
      ENDIF (CMAKE_BUILD_TYPE MATCHES "Release")

      
      # set CXXFLAGS to be fed into CXX_FLAGS by the user:
      IF (OPENSG_OPT)
	SET(OPENSG_CXX_FLAGS "`${OPENSG_CONFIG} --opt --cflags ${OSG_MODULES}`")
      ELSE (OPENSG_OPT)
	SET(OPENSG_CXX_FLAGS "`${OPENSG_CONFIG} --cflags ${OSG_MODULES}`")
      ENDIF (OPENSG_OPT)
	   

      # set INCLUDE_DIRS to prefix+include
      EXEC_PROGRAM(${OPENSG_CONFIG}
	ARGS --prefix
	OUTPUT_VARIABLE OPENSG_PREFIX)
      SET(OPENSG_INCLUDE_DIR ${OPENSG_PREFIX}/include CACHE STRING INTERNAL)
      
      # set link libraries and link flags
      IF (OPENSG_OPT)
	#MESSAGE("DBG using OSG Release opt build")
	SET(OPENSG_LIBRARIES "`${OPENSG_CONFIG} --opt --libs ${OSG_MODULES}`")
      ELSE (OPENSG_OPT)
	#MESSAGE("DBG using OSG debug build")
	SET(OPENSG_LIBRARIES "`${OPENSG_CONFIG} --libs ${OSG_MODULES}`")
      ENDIF (OPENSG_OPT)
      
      ## extract link dirs for rpath  
      EXEC_PROGRAM(${OPENSG_CONFIG}
	ARGS --libs
	OUTPUT_VARIABLE OPENSG_CONFIG_LIBS )

      ## split off the link dirs (for rpath)
      ## use regular expression to match wildcard equivalent "-L*<endchar>"
      ## with <endchar> is a space or a semicolon
      STRING(REGEX MATCHALL "[-][L]([^ ;])+" 
	OPENSG_LINK_DIRECTORIES_WITH_PREFIX 
	"${OPENSG_CONFIG_LIBS}" )
#      MESSAGE("DBG  OPENSG_LINK_DIRECTORIES_WITH_PREFIX=${OPENSG_LINK_DIRECTORIES_WITH_PREFIX}")
      
      ## remove prefix -L because we need the pure directory for LINK_DIRECTORIES
     
      IF (OPENSG_LINK_DIRECTORIES_WITH_PREFIX)
	STRING(REGEX REPLACE "[-][L]" "" OPENSG_LINK_DIRECTORIES ${OPENSG_LINK_DIRECTORIES_WITH_PREFIX} )
      ENDIF (OPENSG_LINK_DIRECTORIES_WITH_PREFIX)
      SET(OPENSG_EXE_LINKER_FLAGS "-Wl,-rpath,${OPENSG_LINK_DIRECTORIES}" CACHE STRING INTERNAL)
#      MESSAGE("DBG  OPENSG_LINK_DIRECTORIES=${OPENSG_LINK_DIRECTORIES}")
#      MESSAGE("DBG  OPENSG_EXE_LINKER_FLAGS=${OPENSG_EXE_LINKER_FLAGS}")


    ELSE(OPENSG_CONFIG)
      MESSAGE("FindOPENSG.cmake: osg-config not found. Please set it manually. OPENSG_CONFIG=${OPENSG_CONFIG}")
    ENDIF(OPENSG_CONFIG)

  ENDIF(UNIX)
ENDIF(WIN32)  ## --------------------------------------------------------------



MARK_AS_ADVANCED(
  OPENSG_CXX_FLAGS
  OPENSG_INCLUDE_DIR
  OPENSG_LIBRARIES
  OPENSG_LINK_DIRECTORIES
  OPENSG_CONFIG
  OPENSG_CONFIG_PREFER_PATH
  OPENSG_EXE_LINKER_FLAGS
)



IF(OPENSG_LIBRARIES)
  IF(OPENSG_INCLUDE_DIR OR OPENSG_CXX_FLAGS)
    
    SET(OPENSG_FOUND ON)
    
    #MESSAGE(STATUS "DBG OPENSG_LIBRARIES=${OPENSG_LIBRARIES}")
    #MESSAGE(STATUS "DBG OPENSG_INCLUDE_DIR=${OPENSG_INCLUDE_DIR}")
    #MESSAGE(STATUS "DBG OPENSG_CXX_FLAGS=${OPENSG_CXX_FLAGS}")
    
  ENDIF(OPENSG_INCLUDE_DIR OR OPENSG_CXX_FLAGS)
ENDIF(OPENSG_LIBRARIES)

