/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "CornerDetectorKLT.hh"
#include <Base/Image/ImageConvert.hh>
#ifdef BIAS_DEBUG
#include <Base/Image/ImageIO.hh>
#endif

  //#define TIME_MEASURE

#include <Base/Debug/TimeMeasure.hh>

#include <limits.h>
#include <algorithm>

using namespace BIAS;
using namespace std;

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class CornerDetectorKLT<unsigned char, float>; 
template class CornerDetectorKLT<float, float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
template class CornerDetectorKLT<unsigned char, int>; 
#endif
#ifdef BUILD_IMAGE_CHAR
#endif
#ifdef BUILD_IMAGE_SHORT
#endif
#ifdef BUILD_IMAGE_USHORT
template class CornerDetectorKLT<unsigned short, float>; 
#ifdef BUILD_IMAGE_INT
template class CornerDetectorKLT<unsigned short, int>; 
#endif
#endif
#ifdef BUILD_IMAGE_UINT
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif

//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////


template <class StorageType, class CalculationType>
CornerDetectorKLT<StorageType, CalculationType>::
CornerDetectorKLT()
  : CornerDetectorGradient<StorageType, CalculationType>()
    
{
}

template <class StorageType, class CalculationType>
CornerDetectorKLT<StorageType, CalculationType>::
CornerDetectorKLT(Param& para, const std::string& postfix)
  : CornerDetectorGradient<StorageType, CalculationType>(para, postfix)
    
{
  //  int grp=para.GetFreeGroupID();
  //_AddParam(para, grp, "-CornerDetectorKLT"+postfix);
  //if (para.IsUsedGroupID(grp)) para.SetGroupName(grp, "CornerDetectorKLT");
}

template <class StorageType, class CalculationType>
CornerDetectorKLT<StorageType, CalculationType>::
CornerDetectorKLT(Param& para, const int grp, const std::string& postfix)
: CornerDetectorGradient<StorageType,CalculationType>(para, grp, postfix)
 
{
  //  _AddParam(para, grp, postfix+"-CornerDetectorKLT");
}

template <class StorageType, class CalculationType>
CornerDetectorKLT<StorageType, CalculationType>::~CornerDetectorKLT()
{}



//////////////////////////////////////////////////////////////////////////////
//                      protected functions
//////////////////////////////////////////////////////////////////////////////


template <class StorageType, class CalculationType>
int CornerDetectorKLT<StorageType, CalculationType>::
_ComputeCornerness(Image<CalculationType>& mineig)
{
#ifdef BIAS_DEBUG
  // _FeatureMap is only allocated when this->_Cornerness is allocated
  BIASASSERT(this->_Cornerness.SamePixelAndChannelCount(this->_sgxx));
  BIASASSERT(this->_Cornerness.SamePixelAndChannelCount(mineig));
#endif
  int tlx, tly, brx, bry;
  this->_sgxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  mineig.GetROI()->SetCorners(tlx, tly, brx, bry);
  const int minx=tlx;
  const int miny=tly;
  const int maxx=brx;
  const int maxy=bry;

  register int x, y;
  register double gxx, gyy, gxy, ev;
  register const double me=(double)*this->_MinCornerness;
  Feat fp; this->_FeatList.clear(); 
  this->_FeatList.reserve((brx-tlx)*(bry-tly));
  register CalculationType **ime=mineig.GetImageDataArray();
  register CalculationType **isgxx=this->_sgxx.GetImageDataArray();
  register CalculationType **isgxy=this->_sgxy.GetImageDataArray();
  register CalculationType **isgyy=this->_sgyy.GetImageDataArray();


  for (y=miny; y<maxy; y++){
    for (x=minx; x<maxx; x++){
      gxx=(double)(isgxx[y][x]);
      gyy=(double)isgyy[y][x];
      gxy=(double)isgxy[y][x];
      ev=((gxx+gyy- sqrt((gxx - gyy)*(gxx - gyy)+4.0*gxy*gxy))/2.0);
      //      cerr << ev<<"\t"<<gxx<<"\t"<<gxy<<"\t"<<gyy<<endl;
      ime[y][x]=(CalculationType)(ev);
      if (ev>me){
        fp.x=x;
        fp.y=y;
        fp.val=(ev>(double)INT_MAX)?(INT_MAX):((int)ev);
        this->_FeatList.push_back(fp);
      }
      /*
#ifdef BIAS_DEBUG
      if (DebugLevelIsSet(D_CD_FEATURES) && y<miny+2 && x<minx+2){
        cerr << "( "<<x<<", "<<y<<") gxx "<<gxx<<"  gxy "<<gxy<<"  gyy "<<gyy
             << "  mineig "<<_Cornerness[y][x]<<endl;
      }
#endif      
      */
    }
  }
#ifdef BIAS_DEBUG
  if (this->DebugLevelIsSet(D_CD_WRITE_DEBUG_IM)){
    ImageIO::Write("mineig.mip", mineig);
  }
#endif
  return 0;
}






