/*
This file is distributed as part of the BIAS library (Basic ImageAlgorithmS)
but it has not been developed by the authors of BIAS.

For copyright, author and license information see below.
*/


/* {{{ Copyright etc. */

/**********************************************************************\

  SUSAN Version 2l by Stephen Smith
  Oxford Centre for Functional Magnetic Resonance Imaging of the Brain,
  Department of Clinical Neurology, Oxford University, Oxford, UK
  (Previously in Computer Vision and Image Processing Group - now
  Computer Vision and Electro Optics Group - DERA Chertsey, UK)
  Email:    steve@fmrib.ox.ac.uk
  WWW:      http://www.fmrib.ox.ac.uk/~steve

  (C) Crown Copyright (1995-1999), Defence Evaluation and Research Agency,
  Farnborough, Hampshire, GU14 6TD, UK
  DERA WWW site:
  http://www.dera.gov.uk/
  DERA Computer Vision and Electro Optics Group WWW site:
  http://www.dera.gov.uk/imageprocessing/dera/group_home.html
  DERA Computer Vision and Electro Optics Group point of contact:
  Dr. John Savage, jtsavage@dera.gov.uk, +44 1344 633203

  A UK patent has been granted: "Method for digitally processing
  images to determine the position of edges and/or corners therein for
  guidance of unmanned vehicle", UK Patent 2272285. Proprietor:
  Secretary of State for Defence, UK. 15 January 1997

  This code is issued for research purposes only and remains the
  property of the UK Secretary of State for Defence. This code must
  not be passed on without this header information being kept
  intact. This code must not be sold.

\**********************************************************************/

/* }}} */
/* {{{ Readme First */

/**********************************************************************\

  SUSAN Version 2l
  SUSAN = Smallest Univalue Segment Assimilating Nucleus

  Email:    steve@fmrib.ox.ac.uk
  WWW:      http://www.fmrib.ox.ac.uk/~steve

  Related paper:
  @article{Smith97,
        author = "Smith, S.M. and Brady, J.M.",
        title = "{SUSAN} - A New Approach to Low Level Image Processing",
        journal = "Int. Journal of Computer Vision",
        pages = "45--78",
        volume = "23",
        number = "1",
        month = "May",
        year = 1997}

  To be registered for automatic (bug) updates of SUSAN, send an email.

  Compile with:
  gcc -O4 -o susan susan2l.c -lm

  See following section for different machine information. Please
  report any bugs (and fixes). There are a few optional changes that
  can be made in the "defines" section which follows shortly.

  Usage: type "susan" to get usage. Only PGM format files can be input
  and output. Utilities such as the netpbm package and XV can be used
  to convert to and from other formats. Any size of image can be
  processed.

  This code is written using an emacs folding mode, making moving
  around the different sections very easy. This is why there are
  various marks within comments and why comments are indented.


  SUSAN QUICK:

  This version of the SUSAN corner finder does not do all the
  false-corner suppression and thus is faster and produced some false
  positives, particularly on strong edges. However, because there are
  less stages involving thresholds etc., the corners that are
  correctly reported are usually more stable than those reported with
  the full algorithm. Thus I recommend at least TRYING this algorithm
  for applications where stability is important, e.g., tracking.

  THRESHOLDS:

  There are two thresholds which can be set at run-time. These are the
  brightness threshold (t) and the distance threshold (d).

  SPATIAL CONTROL: d

  In SUSAN smoothing d controls the size of the Gaussian mask; its
  default is 4.0. Increasing d gives more smoothing. In edge finding,
  a fixed flat mask is used, either 37 pixels arranged in a "circle"
  (default), or a 3 by 3 mask which gives finer detail. In corner
  finding, only the larger 37 pixel mask is used; d is not
  variable. In smoothing, the flat 3 by 3 mask can be used instead of
  a larger Gaussian mask; this gives low smoothing and fast operation.

  BRIGHTNESS CONTROL: t

  In all three algorithms, t can be varied (default=20); this is the
  main threshold to be varied. It determines the maximum difference in
  greylevels between two pixels which allows them to be considered
  part of the same "region" in the image. Thus it can be reduced to
  give more edges or corners, i.e. to be more sensitive, and vice
  versa. In smoothing, reducing t gives less smoothing, and vice
  versa. Set t=10 for the test image available from the SUSAN web
  page.

  ITERATIONS:

  With SUSAN smoothing, more smoothing can also be obtained by
  iterating the algorithm several times. This has a different effect
  from varying d or t.

  FIXED MASKS:

  37 pixel mask:    ooo       3 by 3 mask:  ooo
                   ooooo                    ooo
                  ooooooo                   ooo
                  ooooooo
                  ooooooo
                   ooooo
                    ooo

  CORNER ATTRIBUTES dx, dy and I
  (Only read this if you are interested in the C implementation or in
  using corner attributes, e.g., for corner matching)

  Corners reported in the corner list have attributes associated with
  them as well as positions. This is useful, for example, when
  attempting to match corners from one image to another, as these
  attributes can often be fairly unchanged between images. The
  attributes are dx, dy and I. I is the value of image brightness at
  the position of the corner. In the case of susan_corners_quick, dx
  and dy are the first order derivatives (differentials) of the image
  brightness in the x and y directions respectively, at the position
  of the corner. In the case of normal susan corner finding, dx and dy
  are scaled versions of the position of the centre of gravity of the
  USAN with respect to the centre pixel (nucleus).

  BRIGHTNESS FUNCTION LUT IMPLEMENTATION:
  (Only read this if you are interested in the C implementation)

  The SUSAN brightness function is implemented as a LUT
  (Look-Up-Table) for speed. The resulting pointer-based code is a
  little hard to follow, so here is a brief explanation. In
  setup_brightness_lut() the LUT is setup. This mallocs enough space
  for *bp and then repositions the pointer to the centre of the
  malloced space. The SUSAN function e^-(x^6) or e^-(x^2) is
  calculated and converted to a uchar in the range 0-100, for all
  possible image brightness differences (including negative
  ones). Thus bp[23] is the output for a brightness difference of 23
  greylevels. In the SUSAN algorithms this LUT is used as follows:

  p=in + (i-3)*x_size + j - 1;
  p points to the first image pixel in the circular mask surrounding
  point (x,y).

  cp=bp + in[i*x_size+j];
  cp points to a position in the LUT corresponding to the brightness
  of the centre pixel (x,y).

  now for every pixel within the mask surrounding (x,y),
  n+=*(cp-*p++);
  the brightness difference function is found by moving the cp pointer
  down by an amount equal to the value of the pixel pointed to by p,
  thus subtracting the two brightness values and performing the
  exponential function. This value is added to n, the running USAN
  area.

  in SUSAN smoothing, the variable height mask is implemented by
  multiplying the above by the moving mask pointer, reset for each new
  centre pixel.
  tmp = *dpt++ * *(cp-brightness);

\**********************************************************************/

/* }}} */

#include "CornerDetectorSusan.hh"

#define  FTOI(a) ( (a) < 0 ? ((int)(a-0.5)) : ((int)(a+0.5)) )



using namespace BIAS;
using namespace std;

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class CornerDetectorSusan<unsigned char>; 
//template class CornerDetectorSusan<float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
#endif
#ifdef BUILD_IMAGE_CHAR
#endif
#ifdef BUILD_IMAGE_SHORT
#endif
#ifdef BUILD_IMAGE_USHORT
template class CornerDetectorSusan<unsigned short>; 
#endif
#ifdef BUILD_IMAGE_UINT
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif

//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////


template <class StorageType>
CornerDetectorSusan<StorageType>::
CornerDetectorSusan()
  :   CornerDetectorBase<StorageType>()
{  
  _cgx=_cgy=_r=NULL;
  _bp=NULL;
  _susansize=_susanthresh=0;
  _Threshold=new int;
  *_Threshold = DEFAULT_THRESH; 
  _Quick= new bool;
  *  _Quick= DEFAULT_QUICK; 
  if (this->_MaxNum!=NULL) *this->_MaxNum = 1850;
  this->ParamsAllocated_ = true;
}

template <class StorageType>
CornerDetectorSusan<StorageType>::
CornerDetectorSusan(Param& para, const std::string& postfix)
  :   CornerDetectorBase<StorageType>(para, postfix)
{  
  this->ParamsAllocated_ = false;
  _cgx=_cgy=_r=NULL;
  _bp=NULL;
  _susansize=_susanthresh=0;
  int grp=para.GetFreeGroupID();
  _AddParameter(para, grp, "-CornerDetectorSusan"+postfix);
  if (para.IsUsedGroupID(grp)) para.SetGroupName(grp, "CornerDetectorSusan");
}

template <class StorageType>
CornerDetectorSusan<StorageType>::
CornerDetectorSusan(Param& para, const int grp, const std::string& postfix)
  : CornerDetectorBase<StorageType>(para, grp, postfix) 
{
  this->ParamsAllocated_ = false;
  _cgx=_cgy=_r=NULL;
  _bp=NULL;
  _susansize=_susanthresh=0;
  _AddParameter(para, grp, "-CornerDetectorSusan"+postfix);
}

template <class StorageType>
CornerDetectorSusan<StorageType>::~CornerDetectorSusan()
{
 if (_cgx!=NULL) delete[] _cgx;
  if (_cgy!=NULL) delete[] _cgy;
  if (_r!=NULL) delete[] _r;
  if (_bp!=NULL) delete[] (_bp-258);
  if (this->ParamsAllocated_) {
    delete _Threshold;
    delete _Quick;
  }
}

template <class StorageType>
int CornerDetectorSusan<StorageType>::
Detect(const Image<StorageType>& image, CORNER_LIST corner_list, int& num)
{
#ifdef BIAS_DEBUG
  if ( (image.GetChannelCount()!=1) || (image.IsEmpty()) ) {
    BIASERR("Susan: Can only detect corners on grey value images");
    return -1;
  }
#endif
  int x_size = image.GetWidth();
  int y_size = image.GetHeight();
  if (_susansize<x_size*y_size) {
    _SusanReAllocMem(x_size, y_size);
  }
  StorageType *in = image.GetImageData();
  
  if (_susanthresh!=*_Threshold)
    _setup_brightness_lut(*_Threshold, 6);
  if (*_Quick){
    num = _susan_corners_quick(in, _r, _bp, corner_list, 
                               x_size, y_size);
  } else {
    num = _susan_corners(in, _r, _bp, corner_list, x_size, 
                         y_size);
  }
  BIASDOUT(D_CD_SUSAN,"Susan found "<< num << " corners ");
  return (num>0)?0:-1;
}

template <class StorageType>
int CornerDetectorSusan<StorageType>::
Detect(const Image<StorageType>& image, const Image<StorageType>& roi,
       CORNER_LIST corner_list, int& num)
{
#ifdef BIAS_DEBUG
  if ( (image.GetChannelCount()!=1) || (image.IsEmpty()) ) {
    BIASERR("Susan: Can only detect corners on grey value images");
    return -1;
  }
#endif
  int x_size = image.GetWidth();
  int y_size = image.GetHeight();
  if (_susansize<x_size*y_size) {
    _SusanReAllocMem(x_size, y_size);
  }
  StorageType *in = image.GetImageData();
  StorageType *roid = roi.GetImageData();
  
  if (_susanthresh!=*_Threshold)
    _setup_brightness_lut(*_Threshold, 6);
  if (*_Quick){
    num = _susan_corners_quick(in, _r, _bp, corner_list, 
                               x_size, y_size);
  } else {
    num = _susan_corners(in, _r, _bp, corner_list, x_size, 
                         y_size, roid);
  }
  BIASDOUT(D_CD_SUSAN,"Susan found "<< num << " corners ");
  return (num>0)?0:-1;
}

template <class StorageType>
int CornerDetectorSusan<StorageType>::
Detect(const Image<StorageType>& image, vector<HomgPoint2D>& pvec, 
       vector<float>& quality)
{
#ifdef BIAS_DEBUG
  if ( (image.GetChannelCount()!=1) || (image.IsEmpty()) ) {
    BIASERR("Susan: Can only detect corners on grey value images");
    return -1;
  }
#endif
  int x_size = image.GetWidth();
  int y_size = image.GetHeight();
  if (_susansize<x_size*y_size) {
    _SusanReAllocMem(x_size, y_size);
  }
  StorageType *in = image.GetImageData();
  
  HomgPoint2D p;
  int num;

  pvec.clear();
  quality.clear();

  CORNER_LIST corner_list;

  if (_susanthresh!=*_Threshold)
    _setup_brightness_lut(*_Threshold, 6);
  if (*_Quick)
    num = _susan_corners_quick(in, _r, _bp, corner_list, 
                               x_size, y_size);
  else
    num = _susan_corners(in, _r, _bp, corner_list, 
                         x_size, y_size);
  int n=0;
  quality.resize(num+1);
  pvec.resize(num+1);
  while(corner_list[n].info != 7){
    pvec[n].Set((double)corner_list[n].x, (double)corner_list[n].y);
    n++;
    if (*_Quick){
      /*  In the case of susan_corners_quick, dx
          and dy are the first order derivatives (differentials) of the image
          brightness in the x and y directions respectively, at the position
          of the corner. */
      quality[n]=((float)fabs((float)corner_list[n].dx*corner_list[n].dy)
                  /65025.0f);
      //      cerr << (float)fabs(corner_list[n].dx*corner_list[n].dy)/65025.0 << endl;
    } else {
      /* In the case of normal susan corner finding, dx and dy
         are scaled versions of the position of the centre of gravity of the
         USAN with respect to the centre pixel (nucleus). */
      quality[n]=0.0;
      //cerr << corner_list[n].dx << " " << corner_list[n].dy << endl;
    }
  }
  BIASDOUT(D_CD_SUSAN,"Susan found "<< n << " corners ");
  return 0;
}



template <class StorageType>
void CornerDetectorSusan<StorageType>::
_int_to_uchar(int *r, StorageType *in, int size)
{
  int i,
    max_r=r[0],
    min_r=r[0];
  
  for (i=0; i<size; i++)
    {
      if ( r[i] > max_r )
        max_r=r[i];
      if ( r[i] < min_r )
        min_r=r[i];
    }

  /*printf("min=%d max=%d\n",min_r,max_r);*/

  max_r-=min_r;

  for (i=0; i<size; i++)
    in[i] = (StorageType)((int)((int)(r[i]-min_r)*255)/max_r);
}


template <class StorageType>
void CornerDetectorSusan<StorageType>::
_setup_brightness_lut(int thresh, int form)
{
  int   k;
  float temp;
  
  _susanthresh=thresh;
  if (_bp==NULL){
    _bp=new StorageType[516];
    _bp+=258;
  }
  

  for(k=-256;k<257;k++)
    {
      temp=((float)k)/((float)thresh);
      temp=temp*temp;
      if (form==6)
        temp=temp*temp*temp;
      temp=100.0f*exp(-temp);
      *(_bp+k)= (StorageType)temp;
    }
}

template <class StorageType>
int CornerDetectorSusan<StorageType>::
_susan_corners(StorageType *in, int *r, StorageType *bp,
               CORNER_LIST corner_list, int x_size, int y_size)
{
  const int max_no((*this->_MaxNum>0)? (*this->_MaxNum):INT_MAX);
  int   n,x,y,sq,xx,yy,
    i,j,*cgx,*cgy;
  float divide;
  StorageType c,*p,*cp;

  memset (r,0,x_size * y_size * sizeof(int));

  cgx=new int[x_size*y_size];
  cgy=new int[x_size*y_size];

  for (i=5;i<y_size-5;i++)
    for (j=5;j<x_size-5;j++) {
      n=100;
      p=in + (i-3)*x_size + j - 1;
      cp=bp + in[i*x_size+j];

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-3; 

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-5;

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-6;

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      if (n<max_no){    /* do this test early and often ONLY to save wasted computation */
        p+=2;
        n+=*(cp-*p++);
	if (n<max_no){
	  n+=*(cp-*p++);
	  if (n<max_no){
	    n+=*(cp-*p);
	    if (n<max_no){
	      p+=x_size-6;

	      n+=*(cp-*p++);
	      if (n<max_no){
		n+=*(cp-*p++);
		if (n<max_no){
		  n+=*(cp-*p++);
		  if (n<max_no){
		    n+=*(cp-*p++);
		    if (n<max_no){
		      n+=*(cp-*p++);
		      if (n<max_no){
			n+=*(cp-*p++);
			if (n<max_no){
			  n+=*(cp-*p);
			  if (n<max_no){
			    p+=x_size-5;

			    n+=*(cp-*p++);
			    if (n<max_no){
			      n+=*(cp-*p++);
			      if (n<max_no){
				n+=*(cp-*p++);
				if (n<max_no){
				  n+=*(cp-*p++);
				  if (n<max_no){
				    n+=*(cp-*p);
				    if (n<max_no){
				      p+=x_size-3;

				      n+=*(cp-*p++);
				      if (n<max_no){
					n+=*(cp-*p++);
					if (n<max_no){
					  n+=*(cp-*p);

					  if (n<max_no)
					    {
					      x=0;y=0;
					      p=in + (i-3)*x_size + j - 1;

					      c=*(cp-*p++);x-=c;y-=3*c;
					      c=*(cp-*p++);y-=3*c;
					      c=*(cp-*p);x+=c;y-=3*c;
					      p+=x_size-3; 
    
					      c=*(cp-*p++);x-=2*c;y-=2*c;
					      c=*(cp-*p++);x-=c;y-=2*c;
					      c=*(cp-*p++);y-=2*c;
					      c=*(cp-*p++);x+=c;y-=2*c;
					      c=*(cp-*p);x+=2*c;y-=2*c;
					      p+=x_size-5;
    
					      c=*(cp-*p++);x-=3*c;y-=c;
					      c=*(cp-*p++);x-=2*c;y-=c;
					      c=*(cp-*p++);x-=c;y-=c;
					      c=*(cp-*p++);y-=c;
					      c=*(cp-*p++);x+=c;y-=c;
					      c=*(cp-*p++);x+=2*c;y-=c;
					      c=*(cp-*p);x+=3*c;y-=c;
					      p+=x_size-6;

					      c=*(cp-*p++);x-=3*c;
					      c=*(cp-*p++);x-=2*c;
					      c=*(cp-*p);x-=c;
					      p+=2;
					      c=*(cp-*p++);x+=c;
					      c=*(cp-*p++);x+=2*c;
					      c=*(cp-*p);x+=3*c;
					      p+=x_size-6;
    
					      c=*(cp-*p++);x-=3*c;y+=c;
					      c=*(cp-*p++);x-=2*c;y+=c;
					      c=*(cp-*p++);x-=c;y+=c;
					      c=*(cp-*p++);y+=c;
					      c=*(cp-*p++);x+=c;y+=c;
					      c=*(cp-*p++);x+=2*c;y+=c;
					      c=*(cp-*p);x+=3*c;y+=c;
					      p+=x_size-5;

					      c=*(cp-*p++);x-=2*c;y+=2*c;
					      c=*(cp-*p++);x-=c;y+=2*c;
					      c=*(cp-*p++);y+=2*c;
					      c=*(cp-*p++);x+=c;y+=2*c;
					      c=*(cp-*p);x+=2*c;y+=2*c;
					      p+=x_size-3;

					      c=*(cp-*p++);x-=c;y+=3*c;
					      c=*(cp-*p++);y+=3*c;
					      c=*(cp-*p);x+=c;y+=3*c;

					      xx=x*x;
					      yy=y*y;
					      sq=xx+yy;
					      if ( sq > ((n*n)/2) )
						{
						  if(yy<xx) {
						    divide=(float)y/(float)abs(x);
						    sq=abs(x)/x;
						    sq=*(cp-in[(i+FTOI(divide))*x_size+j+sq]) +
						      *(cp-in[(i+FTOI(2*divide))*x_size+j+2*sq]) +
						      *(cp-in[(i+FTOI(3*divide))*x_size+j+3*sq]);}
						  else {
						    divide=(float)x/(float)abs(y);
						    sq=abs(y)/y;
						    sq=*(cp-in[(i+sq)*x_size+j+FTOI(divide)]) +
						      *(cp-in[(i+2*sq)*x_size+j+FTOI(2*divide)]) +
						      *(cp-in[(i+3*sq)*x_size+j+FTOI(3*divide)]);}

						  if(sq>290){
						    r[i*x_size+j] = max_no-n;
						    cgx[i*x_size+j] = (51*x)/n;
						    cgy[i*x_size+j] = (51*y)/n;}
						}
					    }
					}}}}}}}}}}}}}}}}}}}

  /* to locate the local maxima */
  n=0;
  for (i=5;i<y_size-5;i++)
    for (j=5;j<x_size-5;j++) {
      x = r[i*x_size+j];
      if (x>0)  {
	/* 5x5 mask */
#ifdef FIVE_SUPP
	if (
	    (x>r[(i-1)*x_size+j+2]) &&
	    (x>r[(i  )*x_size+j+1]) &&
	    (x>r[(i  )*x_size+j+2]) &&
	    (x>r[(i+1)*x_size+j-1]) &&
	    (x>r[(i+1)*x_size+j  ]) &&
	    (x>r[(i+1)*x_size+j+1]) &&
	    (x>r[(i+1)*x_size+j+2]) &&
	    (x>r[(i+2)*x_size+j-2]) &&
	    (x>r[(i+2)*x_size+j-1]) &&
	    (x>r[(i+2)*x_size+j  ]) &&
	    (x>r[(i+2)*x_size+j+1]) &&
	    (x>r[(i+2)*x_size+j+2]) &&
	    (x>=r[(i-2)*x_size+j-2]) &&
	    (x>=r[(i-2)*x_size+j-1]) &&
	    (x>=r[(i-2)*x_size+j  ]) &&
	    (x>=r[(i-2)*x_size+j+1]) &&
	    (x>=r[(i-2)*x_size+j+2]) &&
	    (x>=r[(i-1)*x_size+j-2]) &&
	    (x>=r[(i-1)*x_size+j-1]) &&
	    (x>=r[(i-1)*x_size+j  ]) &&
	    (x>=r[(i-1)*x_size+j+1]) &&
	    (x>=r[(i  )*x_size+j-2]) &&
	    (x>=r[(i  )*x_size+j-1]) &&
	    (x>=r[(i+1)*x_size+j-2]) )
#endif
#ifdef SEVEN_SUPP
          if ( 
	      (x>r[(i-3)*x_size+j-3]) &&
	      (x>r[(i-3)*x_size+j-2]) &&
	      (x>r[(i-3)*x_size+j-1]) &&
	      (x>r[(i-3)*x_size+j  ]) &&
	      (x>r[(i-3)*x_size+j+1]) &&
	      (x>r[(i-3)*x_size+j+2]) &&
	      (x>r[(i-3)*x_size+j+3]) &&

	      (x>r[(i-2)*x_size+j-3]) &&
	      (x>r[(i-2)*x_size+j-2]) &&
	      (x>r[(i-2)*x_size+j-1]) &&
	      (x>r[(i-2)*x_size+j  ]) &&
	      (x>r[(i-2)*x_size+j+1]) &&
	      (x>r[(i-2)*x_size+j+2]) &&
	      (x>r[(i-2)*x_size+j+3]) &&

	      (x>r[(i-1)*x_size+j-3]) &&
	      (x>r[(i-1)*x_size+j-2]) &&
	      (x>r[(i-1)*x_size+j-1]) &&
	      (x>r[(i-1)*x_size+j  ]) &&
	      (x>r[(i-1)*x_size+j+1]) &&
	      (x>r[(i-1)*x_size+j+2]) &&
	      (x>r[(i-1)*x_size+j+3]) &&

	      (x>r[(i)*x_size+j-3]) &&
	      (x>r[(i)*x_size+j-2]) &&
	      (x>r[(i)*x_size+j-1]) &&
	      (x>=r[(i)*x_size+j+1]) &&
	      (x>=r[(i)*x_size+j+2]) &&
	      (x>=r[(i)*x_size+j+3]) &&

	      (x>=r[(i+1)*x_size+j-3]) &&
	      (x>=r[(i+1)*x_size+j-2]) &&
	      (x>=r[(i+1)*x_size+j-1]) &&
	      (x>=r[(i+1)*x_size+j  ]) &&
	      (x>=r[(i+1)*x_size+j+1]) &&
	      (x>=r[(i+1)*x_size+j+2]) &&
	      (x>=r[(i+1)*x_size+j+3]) &&

	      (x>=r[(i+2)*x_size+j-3]) &&
	      (x>=r[(i+2)*x_size+j-2]) &&
	      (x>=r[(i+2)*x_size+j-1]) &&
	      (x>=r[(i+2)*x_size+j  ]) &&
	      (x>=r[(i+2)*x_size+j+1]) &&
	      (x>=r[(i+2)*x_size+j+2]) &&
	      (x>=r[(i+2)*x_size+j+3]) &&

	      (x>=r[(i+3)*x_size+j-3]) &&
	      (x>=r[(i+3)*x_size+j-2]) &&
	      (x>=r[(i+3)*x_size+j-1]) &&
	      (x>=r[(i+3)*x_size+j  ]) &&
	      (x>=r[(i+3)*x_size+j+1]) &&
	      (x>=r[(i+3)*x_size+j+2]) &&
	      (x>=r[(i+3)*x_size+j+3]) )
#endif
	    {
	      corner_list[n].info=0;
	      corner_list[n].x=j;
	      corner_list[n].y=i;
	      corner_list[n].dx=cgx[i*x_size+j];
	      corner_list[n].dy=cgy[i*x_size+j];
	      corner_list[n].I=in[i*x_size+j];
	      n++;
	      if(n==MAX_CORNERS){
          BIASERR("Too many corners.\n");
          return n;
	      }}}}
  corner_list[n].info=7;

  delete[] cgx;
  delete[] cgy;
  return n;
}

/* some modifications by woelk
   - added roi to avoid unnecessary computqations
   - introduced xj* to avoid unnecessary computations
   - introduced _cgx, _cgy and _susanwidth, _susanheight to avoid unnecessaity 
       malloc/free */
template <class StorageType>
int CornerDetectorSusan<StorageType>::
_susan_corners(StorageType *in, int *r, StorageType *bp, 
               CORNER_LIST corner_list, int x_size, int y_size, 
               StorageType *roi) { 
  const int max_no((*this->_MaxNum>0)? (*this->_MaxNum):INT_MAX);
  int   n,x,y,sq,xx,yy,i,j;
  float divide;
  StorageType c, *p, *cp;
  int maxx=x_size-5, maxy=y_size-5;

#ifdef FIVE_SUPP
  int j1p, j1m, j2p, j2m, xi, xi1m, xi1p, xi2m, xi2p;
#endif
#ifdef SEVEN_SUPP
  int j1p, j1m, j2p, j2m, j3p, j3m, xi, xi1m, xi1p, xi2m, xi2p, xi3m, xi3p;
#endif

  memset (r,0,x_size * y_size * sizeof(int));
  
 
  if (_susansize<x_size*y_size){
    _SusanReAllocMem(x_size, y_size);
  }
  
  for (i=5;i<maxy;i++)
    for (j=5;j<maxx;j++) {
      if (roi[i*x_size+j]!=0){
        n=100;
        p=in + (i-3)*x_size + j - 1;
        cp=bp + in[i*x_size+j];

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        p+=x_size-3; 

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        p+=x_size-5;

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        p+=x_size-6;

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        if (n<max_no){    /* do this test early and often ONLY to save wasted computation */
          p+=2;
          n+=*(cp-*p++);
          if (n<max_no){
            n+=*(cp-*p++);
            if (n<max_no){
              n+=*(cp-*p);
              if (n<max_no){
                p+=x_size-6;

                n+=*(cp-*p++);
                if (n<max_no){
                  n+=*(cp-*p++);
                  if (n<max_no){
                    n+=*(cp-*p++);
                    if (n<max_no){
                      n+=*(cp-*p++);
                      if (n<max_no){
                        n+=*(cp-*p++);
                        if (n<max_no){
                          n+=*(cp-*p++);
                          if (n<max_no){
                            n+=*(cp-*p);
                            if (n<max_no){
                              p+=x_size-5;

                              n+=*(cp-*p++);
                              if (n<max_no){
                                n+=*(cp-*p++);
                                if (n<max_no){
                                  n+=*(cp-*p++);
                                  if (n<max_no){
                                    n+=*(cp-*p++);
                                    if (n<max_no){
                                      n+=*(cp-*p);
                                      if (n<max_no){
                                        p+=x_size-3;

                                        n+=*(cp-*p++);
                                        if (n<max_no){
                                          n+=*(cp-*p++);
                                          if (n<max_no){
                                            n+=*(cp-*p);

                                            if (n<max_no)
                                              {
                                                x=0;y=0;
                                                p=in + (i-3)*x_size + j - 1;

                                                c=*(cp-*p++);x-=c;y-=3*c;
                                                c=*(cp-*p++);y-=3*c;
                                                c=*(cp-*p);x+=c;y-=3*c;
                                                p+=x_size-3; 
    
                                                c=*(cp-*p++);x-=2*c;y-=2*c;
                                                c=*(cp-*p++);x-=c;y-=2*c;
                                                c=*(cp-*p++);y-=2*c;
                                                c=*(cp-*p++);x+=c;y-=2*c;
                                                c=*(cp-*p);x+=2*c;y-=2*c;
                                                p+=x_size-5;
    
                                                c=*(cp-*p++);x-=3*c;y-=c;
                                                c=*(cp-*p++);x-=2*c;y-=c;
                                                c=*(cp-*p++);x-=c;y-=c;
                                                c=*(cp-*p++);y-=c;
                                                c=*(cp-*p++);x+=c;y-=c;
                                                c=*(cp-*p++);x+=2*c;y-=c;
                                                c=*(cp-*p);x+=3*c;y-=c;
                                                p+=x_size-6;

                                                c=*(cp-*p++);x-=3*c;
                                                c=*(cp-*p++);x-=2*c;
                                                c=*(cp-*p);x-=c;
                                                p+=2;
                                                c=*(cp-*p++);x+=c;
                                                c=*(cp-*p++);x+=2*c;
                                                c=*(cp-*p);x+=3*c;
                                                p+=x_size-6;
    
                                                c=*(cp-*p++);x-=3*c;y+=c;
                                                c=*(cp-*p++);x-=2*c;y+=c;
                                                c=*(cp-*p++);x-=c;y+=c;
                                                c=*(cp-*p++);y+=c;
                                                c=*(cp-*p++);x+=c;y+=c;
                                                c=*(cp-*p++);x+=2*c;y+=c;
                                                c=*(cp-*p);x+=3*c;y+=c;
                                                p+=x_size-5;

                                                c=*(cp-*p++);x-=2*c;y+=2*c;
                                                c=*(cp-*p++);x-=c;y+=2*c;
                                                c=*(cp-*p++);y+=2*c;
                                                c=*(cp-*p++);x+=c;y+=2*c;
                                                c=*(cp-*p);x+=2*c;y+=2*c;
                                                p+=x_size-3;

                                                c=*(cp-*p++);x-=c;y+=3*c;
                                                c=*(cp-*p++);y+=3*c;
                                                c=*(cp-*p);x+=c;y+=3*c;

                                                xx=x*x;
                                                yy=y*y;
                                                sq=xx+yy;
                                                if ( sq > ((n*n)/2) ){
                                                  if(yy<xx) {
                                                    divide=(float)y/(float)abs(x);
                                                    sq=abs(x)/x;
                                                    sq=*(cp-in[(i+FTOI(divide))*x_size+j+sq]) +
                                                      *(cp-in[(i+FTOI(2*divide))*x_size+j+2*sq]) +
                                                      *(cp-in[(i+FTOI(3*divide))*x_size+j+3*sq]);}
                                                  else {
                                                    divide=(float)x/(float)abs(y);
                                                    sq=abs(y)/y;
                                                    sq=*(cp-in[(i+sq)*x_size+j+FTOI(divide)]) +
                                                      *(cp-in[(i+2*sq)*x_size+j+FTOI(2*divide)]) +
                                                      *(cp-in[(i+3*sq)*x_size+j+FTOI(3*divide)]);}
                                                
                                                  if(sq>290){
                                                    r[i*x_size+j] = max_no-n;
                                                    _cgx[i*x_size+j] = (51*x)/n;
                                                    _cgy[i*x_size+j] = (51*y)/n;}
                                                }
                                              }
                                          }}}}}}}}}}}}}}}}}}}}

  /* to locate the local maxima */
  n=0;
  for (i=5;i<maxy;i++)
    for (j=5;j<maxx;j++) {
      x = r[i*x_size+j];
      if (x>0)  {
        j1p=j+1;
        j2p=j+2;
        j1m=j-1;
        j2m=j-2;
        xi=i*x_size;
        xi1m=(i-1)*x_size;
        xi1p=(i+1)*x_size;
        xi2m=(i-2)*x_size;
        xi2p=(i+2)*x_size;
        /* 5x5 mask */
#ifdef FIVE_SUPP
        if (
            (x>r[xi1m+j2p]) &&
            (x>r[xi+j1p]) &&
            (x>r[xi+j2p]) &&
            (x>r[xi1p+j1m]) &&
            (x>r[xi1p+j  ]) &&
            (x>r[xi1p+j1p]) &&
            (x>r[xi1p+j2p]) &&
            (x>r[xi2p+j2m]) &&
            (x>r[xi2p+j1m]) &&
            (x>r[xi2p+j  ]) &&
            (x>r[xi2p+j1p]) &&
            (x>r[xi2p+j2p]) &&
            (x>=r[xi2m+j2m]) &&
            (x>=r[xi2m+j1m]) &&
            (x>=r[xi2m+j  ]) &&
            (x>=r[xi2m+j1p]) &&
            (x>=r[xi2m+j2p]) &&
            (x>=r[xi1m+j2m]) &&
            (x>=r[xi1m+j1m]) &&
            (x>=r[xi1m+j  ]) &&
            (x>=r[xi1m+j1p]) &&
            (x>=r[xi+j2m]) &&
            (x>=r[xi+j1m]) &&
            (x>=r[xi1p+j2m]) )
#endif
#ifdef SEVEN_SUPP
        j3p=j+3;
        j3m=j-3;
        xi3m=(i-3)*x_size;
        xi3p=(i+3)*x_size;
        if ( 
            (x>r[xi3m+j3m]) &&
            (x>r[xi3m+j2m]) &&
            (x>r[xi3m+j1m]) &&
            (x>r[xi3m+j  ]) &&
            (x>r[xi3m+j1p]) &&
            (x>r[xi3m+j2p]) &&
            (x>r[xi3m+j3p]) &&

            (x>r[xi2m+j3m]) &&
            (x>r[xi2m+j2m]) &&
            (x>r[xi2m+j1m]) &&
            (x>r[xi2m+j  ]) &&
            (x>r[xi2m+j1p]) &&
            (x>r[xi2m+j2p]) &&
            (x>r[xi2m+j3p]) &&

            (x>r[xi1m+j3m]) &&
            (x>r[xi1m+j2m]) &&
            (x>r[xi1m+j1m]) &&
            (x>r[xi1m+j  ]) &&
            (x>r[xi1m+j1p]) &&
            (x>r[xi1m+j2p]) &&
            (x>r[xi1m+j3p]) &&

            (x>r[xi+j3m]) &&
            (x>r[xi+j2m]) &&
            (x>r[xi+j1m]) &&
            (x>=r[xi+j1p]) &&
            (x>=r[xi+j2p]) &&
            (x>=r[xi+j3p]) &&

            (x>=r[xi1p+j3m]) &&
            (x>=r[xi1p+j2m]) &&
            (x>=r[xi1p+j1m]) &&
            (x>=r[xi1p+j  ]) &&
            (x>=r[xi1p+j1p]) &&
            (x>=r[xi1p+j2p]) &&
            (x>=r[xi1p+j3p]) &&

            (x>=r[xi2p+j3m]) &&
            (x>=r[xi2p+j2m]) &&
            (x>=r[xi2p+j1m]) &&
            (x>=r[xi2p+j  ]) &&
            (x>=r[xi2p+j1p]) &&
            (x>=r[xi2p+j2p]) &&
            (x>=r[xi2p+j3p]) &&

            (x>=r[xi3p+j3m]) &&
            (x>=r[xi3p+j2m]) &&
            (x>=r[xi3p+j1m]) &&
            (x>=r[xi3p+j  ]) &&
            (x>=r[xi3p+j1p]) &&
            (x>=r[xi3p+j2p]) &&
            (x>=r[xi3p+j3p]) )
#endif
          {
            corner_list[n].info=0;
            corner_list[n].x=j;
            corner_list[n].y=i;
            corner_list[n].dx=_cgx[i*x_size+j];
            corner_list[n].dy=_cgy[i*x_size+j];
            corner_list[n].I=in[i*x_size+j];
            n++;
            if(n==MAX_CORNERS){
              BIASERR("Too many corners.\n");
              return n;
            }}}}
  corner_list[n].info=7;

  return n;
}

template <class StorageType>
int CornerDetectorSusan<StorageType>::
_susan_corners_quick(StorageType *in, int *r, StorageType *bp, 
                     CORNER_LIST corner_list, int x_size, int y_size)
{
  const int max_no((*this->_MaxNum>0)? (*this->_MaxNum):INT_MAX);
  int   n,x,y,i,j;
  StorageType *p, *cp;

  memset (r,0,x_size * y_size * sizeof(int));

  for (i=7;i<y_size-7;i++)
    for (j=7;j<x_size-7;j++) {
      n=100;
      p=in + (i-3)*x_size + j - 1;
      cp=bp + in[i*x_size+j];

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-3;

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-5;

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      p+=x_size-6;

      n+=*(cp-*p++);
      n+=*(cp-*p++);
      n+=*(cp-*p);
      if (n<max_no){
        p+=2;
        n+=*(cp-*p++);
	if (n<max_no){
	  n+=*(cp-*p++);
	  if (n<max_no){
	    n+=*(cp-*p);
	    if (n<max_no){
	      p+=x_size-6;

	      n+=*(cp-*p++);
	      if (n<max_no){
		n+=*(cp-*p++);
		if (n<max_no){
		  n+=*(cp-*p++);
		  if (n<max_no){
		    n+=*(cp-*p++);
		    if (n<max_no){
		      n+=*(cp-*p++);
		      if (n<max_no){
			n+=*(cp-*p++);
			if (n<max_no){
			  n+=*(cp-*p);
			  if (n<max_no){
			    p+=x_size-5;

			    n+=*(cp-*p++);
			    if (n<max_no){
			      n+=*(cp-*p++);
			      if (n<max_no){
				n+=*(cp-*p++);
				if (n<max_no){
				  n+=*(cp-*p++);
				  if (n<max_no){
				    n+=*(cp-*p);
				    if (n<max_no){
				      p+=x_size-3;

				      n+=*(cp-*p++);
				      if (n<max_no){
					n+=*(cp-*p++);
					if (n<max_no){
					  n+=*(cp-*p);

					  if (n<max_no)
					    r[i*x_size+j] = max_no-n;
					}}}}}}}}}}}}}}}}}}}

  /* to locate the local maxima */
  n=0;
  for (i=7;i<y_size-7;i++)
    for (j=7;j<x_size-7;j++) {
      x = r[i*x_size+j];
      if (x>0)  {
	/* 5x5 mask */
#ifdef FIVE_SUPP
	if (
	    (x>r[(i-1)*x_size+j+2]) &&
	    (x>r[(i  )*x_size+j+1]) &&
	    (x>r[(i  )*x_size+j+2]) &&
	    (x>r[(i+1)*x_size+j-1]) &&
	    (x>r[(i+1)*x_size+j  ]) &&
	    (x>r[(i+1)*x_size+j+1]) &&
	    (x>r[(i+1)*x_size+j+2]) &&
	    (x>r[(i+2)*x_size+j-2]) &&
	    (x>r[(i+2)*x_size+j-1]) &&
	    (x>r[(i+2)*x_size+j  ]) &&
	    (x>r[(i+2)*x_size+j+1]) &&
	    (x>r[(i+2)*x_size+j+2]) &&
	    (x>=r[(i-2)*x_size+j-2]) &&
	    (x>=r[(i-2)*x_size+j-1]) &&
	    (x>=r[(i-2)*x_size+j  ]) &&
	    (x>=r[(i-2)*x_size+j+1]) &&
	    (x>=r[(i-2)*x_size+j+2]) &&
	    (x>=r[(i-1)*x_size+j-2]) &&
	    (x>=r[(i-1)*x_size+j-1]) &&
	    (x>=r[(i-1)*x_size+j  ]) &&
	    (x>=r[(i-1)*x_size+j+1]) &&
	    (x>=r[(i  )*x_size+j-2]) &&
	    (x>=r[(i  )*x_size+j-1]) &&
	    (x>=r[(i+1)*x_size+j-2]) )
#endif
#ifdef SEVEN_SUPP
          if ( 
	      (x>r[(i-3)*x_size+j-3]) &&
	      (x>r[(i-3)*x_size+j-2]) &&
	      (x>r[(i-3)*x_size+j-1]) &&
	      (x>r[(i-3)*x_size+j  ]) &&
	      (x>r[(i-3)*x_size+j+1]) &&
	      (x>r[(i-3)*x_size+j+2]) &&
	      (x>r[(i-3)*x_size+j+3]) &&

	      (x>r[(i-2)*x_size+j-3]) &&
	      (x>r[(i-2)*x_size+j-2]) &&
	      (x>r[(i-2)*x_size+j-1]) &&
	      (x>r[(i-2)*x_size+j  ]) &&
	      (x>r[(i-2)*x_size+j+1]) &&
	      (x>r[(i-2)*x_size+j+2]) &&
	      (x>r[(i-2)*x_size+j+3]) &&

	      (x>r[(i-1)*x_size+j-3]) &&
	      (x>r[(i-1)*x_size+j-2]) &&
	      (x>r[(i-1)*x_size+j-1]) &&
	      (x>r[(i-1)*x_size+j  ]) &&
	      (x>r[(i-1)*x_size+j+1]) &&
	      (x>r[(i-1)*x_size+j+2]) &&
	      (x>r[(i-1)*x_size+j+3]) &&

	      (x>r[(i)*x_size+j-3]) &&
	      (x>r[(i)*x_size+j-2]) &&
	      (x>r[(i)*x_size+j-1]) &&
	      (x>=r[(i)*x_size+j+1]) &&
	      (x>=r[(i)*x_size+j+2]) &&
	      (x>=r[(i)*x_size+j+3]) &&

	      (x>=r[(i+1)*x_size+j-3]) &&
	      (x>=r[(i+1)*x_size+j-2]) &&
	      (x>=r[(i+1)*x_size+j-1]) &&
	      (x>=r[(i+1)*x_size+j  ]) &&
	      (x>=r[(i+1)*x_size+j+1]) &&
	      (x>=r[(i+1)*x_size+j+2]) &&
	      (x>=r[(i+1)*x_size+j+3]) &&

	      (x>=r[(i+2)*x_size+j-3]) &&
	      (x>=r[(i+2)*x_size+j-2]) &&
	      (x>=r[(i+2)*x_size+j-1]) &&
	      (x>=r[(i+2)*x_size+j  ]) &&
	      (x>=r[(i+2)*x_size+j+1]) &&
	      (x>=r[(i+2)*x_size+j+2]) &&
	      (x>=r[(i+2)*x_size+j+3]) &&

	      (x>=r[(i+3)*x_size+j-3]) &&
	      (x>=r[(i+3)*x_size+j-2]) &&
	      (x>=r[(i+3)*x_size+j-1]) &&
	      (x>=r[(i+3)*x_size+j  ]) &&
	      (x>=r[(i+3)*x_size+j+1]) &&
	      (x>=r[(i+3)*x_size+j+2]) &&
	      (x>=r[(i+3)*x_size+j+3]) )
#endif
	    {
	      corner_list[n].info=0;
	      corner_list[n].x=j;
	      corner_list[n].y=i;
	      x = in[(i-2)*x_size+j-2] + in[(i-2)*x_size+j-1] + in[(i-2)*x_size+j] + in[(i-2)*x_size+j+1] + in[(i-2)*x_size+j+2] +
		in[(i-1)*x_size+j-2] + in[(i-1)*x_size+j-1] + in[(i-1)*x_size+j] + in[(i-1)*x_size+j+1] + in[(i-1)*x_size+j+2] +
		in[(i  )*x_size+j-2] + in[(i  )*x_size+j-1] + in[(i  )*x_size+j] + in[(i  )*x_size+j+1] + in[(i  )*x_size+j+2] +
		in[(i+1)*x_size+j-2] + in[(i+1)*x_size+j-1] + in[(i+1)*x_size+j] + in[(i+1)*x_size+j+1] + in[(i+1)*x_size+j+2] +
		in[(i+2)*x_size+j-2] + in[(i+2)*x_size+j-1] + in[(i+2)*x_size+j] + in[(i+2)*x_size+j+1] + in[(i+2)*x_size+j+2];

	      corner_list[n].I=x/25;
	      /*corner_list[n].I=in[i*x_size+j];*/
	      x = in[(i-2)*x_size+j+2] + in[(i-1)*x_size+j+2] + in[(i)*x_size+j+2] + in[(i+1)*x_size+j+2] + in[(i+2)*x_size+j+2] -
		(in[(i-2)*x_size+j-2] + in[(i-1)*x_size+j-2] + in[(i)*x_size+j-2] + in[(i+1)*x_size+j-2] + in[(i+2)*x_size+j-2]);
	      x += x + in[(i-2)*x_size+j+1] + in[(i-1)*x_size+j+1] + in[(i)*x_size+j+1] + in[(i+1)*x_size+j+1] + in[(i+2)*x_size+j+1] -
		(in[(i-2)*x_size+j-1] + in[(i-1)*x_size+j-1] + in[(i)*x_size+j-1] + in[(i+1)*x_size+j-1] + in[(i+2)*x_size+j-1]);

	      y = in[(i+2)*x_size+j-2] + in[(i+2)*x_size+j-1] + in[(i+2)*x_size+j] + in[(i+2)*x_size+j+1] + in[(i+2)*x_size+j+2] -
		(in[(i-2)*x_size+j-2] + in[(i-2)*x_size+j-1] + in[(i-2)*x_size+j] + in[(i-2)*x_size+j+1] + in[(i-2)*x_size+j+2]);
	      y += y + in[(i+1)*x_size+j-2] + in[(i+1)*x_size+j-1] + in[(i+1)*x_size+j] + in[(i+1)*x_size+j+1] + in[(i+1)*x_size+j+2] -
		(in[(i-1)*x_size+j-2] + in[(i-1)*x_size+j-1] + in[(i-1)*x_size+j] + in[(i-1)*x_size+j+1] + in[(i-1)*x_size+j+2]);
	      corner_list[n].dx=x/15;
	      corner_list[n].dy=y/15;
//  	      point.Set((double)j, (double)i);
//  	      pvec.push_back(point);
//  	      quality.push_back((float)(x/15*y/15));

	      n++;
	      if(n==MAX_CORNERS){
		BIASERR("Too many corners.\n");
    return n;
	      }}}}
  corner_list[n].info=7;
  return n;
}

template <class StorageType>
void CornerDetectorSusan<StorageType>::
_AddParameter(Param& para, const int grp, const std::string& postfix)
{
  BIASCDOUT(D_CD_SUSAN, "Tracker<StorageType>: using group ID "
            <<grp<<"\n");

  if (para.CheckParam("Threshold"+postfix)){ 
    _Threshold=para.GetParamInt("Threshold"+postfix);
  } else { 
    _Threshold=para.AddParamInt("Threshold"+postfix, 
                                "threshold for the maximum difference in greylevels between two pixels which allows them to be considered part of the same \"region\" in the image", 
                                DEFAULT_THRESH, 0, 100, 't', grp); 
  }

  if (para.CheckParam("Quick"+postfix)){ 
    _Quick=para.GetParamBool("Quick"+postfix);
  } else { 
    _Quick=para.AddParamBool("Quick"+postfix, 
                             "should the quick mode be used", 
                             DEFAULT_QUICK, 'q', grp); 
  }
  if (this->_MaxNum!=NULL) *this->_MaxNum = 1850;
}

/* {{{ Machine Information */

/**********************************************************************\

  Success has been reported with the following:

  MACHINE  OS         COMPILER

  Sun      4.1.4      bundled C, gcc

  Next

  SGI      IRIX       SGI cc

  DEC      Unix V3.2+ 

  IBM RISC AIX        gcc

  PC                  Borland 5.0

  PC       Linux      gcc-2.6.3

  PC       Win32      Visual C++ 4.0 (Console Application)

  PC       Win95      Visual C++ 5.0 (Console Application)
                      Thanks to Niu Yongsheng <niuysbit@163.net>:
                      Use the FOPENB option below

  PC       DOS        djgpp gnu C
                      Thanks to Mark Pettovello <mpettove@umdsun2.umd.umich.edu>:
                      Use the FOPENB option below

  HP       HP-UX      bundled cc
                      Thanks to Brian Dixon <briand@hpcvsgen.cv.hp.com>:
                      in ksh:
                      export CCOPTS="-Aa -D_HPUX_SOURCE | -lM"
                      cc -O3 -o susan susan2l.c

\**********************************************************************/

/* }}} */
