/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2004
  Johan Skoglund
  skoglund@isy.liu.se
  CVL/isy
  university of Linkping
  sweden

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Filter/Gauss.hh"
#include "Base/Image/ImageIO.hh"
#include <iostream>

#include <Base/Debug/TimeMeasure.hh>

using namespace BIAS;
using namespace std;

#define TYPE2 unsigned char


int main(int argc, char *argv[])
{
  TimeMeasure timeri, timerf, timer7;

  Image<unsigned char> src; 
  if (ImageIO::Load(argv[1], src)!=0){
    cout <<"Error loading "<<argv[1]<<endl;
    exit(-1);
  }
  
  Image<TYPE2> dst, dst1, dst2, fdiff;

  // input image is unsigned char, output is float !
  Gauss<unsigned char, TYPE2> theGauss;
  
  theGauss.AddDebugLevel(D_CONV_TYPES);
  theGauss.AddDebugLevel(D_WRITE_IMAGES);
  theGauss.AddDebugLevel(D_FILTERBASE_CALLSTACK);
  theGauss.AddDebugLevel(D_CONV_KERNEL); 

  
  cout<<"now filtering in 7x7"<<endl;
  theGauss.FilterFloat(src, dst2);
  ImageIO::Write("7x7_fitered", dst2);
  cout<<"now filtering in int"<<endl;
  theGauss.FilterInt(src, dst1);
  ImageIO::Write("int_filtered", dst1);
  
  cout<<"now filtering in float"<<endl;
  theGauss.FilterFloat(src, dst2);
  ImageIO::Write("float_fitered", dst2);

 
  theGauss.Filter(src, dst);
  ImageIO::Write( "auto_filtered", dst);
  dst.SetZero(); 

  bool WantCompare = true;
  if (WantCompare) {
    TYPE2 fmin, fmax;
    fdiff.AbsDiff(dst1, dst2);  
    fdiff.GetMinMaxPixelValue(fmin, fmax);
    cerr << "differences between "<<(double)fmin<<" and "<<(double)fmax<<endl;
  }
  bool WantTiming = true;
  if (WantTiming) {
    theGauss.SetDebugLevel(0);
    // warming up
    for (int i=0; i<10; i++){
      cout<<"now filtering in int"<<endl;
      theGauss.FilterInt(src, dst1);
 
      cout<<"now filtering in float"<<endl;
      theGauss.FilterFloat(src, dst2);
    }
    cout<<"Now comparing performance..."<<endl<<flush;
    // go
    for (int i=0; i<100; i++){
      timeri.Start();
      theGauss.FilterInt(src, dst1);
      timeri.Stop();

      timerf.Start();
      theGauss.FilterFloat(src, dst2);
      timerf.Stop();

      timer7.Start();
      theGauss.Filter7x7Grey(src, dst2);
      timer7.Stop();

    }

    cerr << "int took "<<endl;
    timeri.Print();

    cerr << "float took "<<endl;
    timerf.Print();
    
    cerr << "7x7 took "<<endl;
    timer7.Print();
  
  }
  return 0;
}
