#include "ImageCanvas.hh"

#include <Base/Image/ImageConvert.hh>

#ifdef   WIN32
#include <wx/image.h>
#endif //WIN32

#include <sstream>
#include <iomanip>

using namespace BIAS;
using namespace std;

#define SCROLLBAR_WIDTH 15

/*
#undef BIAS_MUTEX_LOCK
#define BIAS_MUTEX_LOCK(arg)\
      cerr <<__FILE__<<":"<<__LINE__<<" attempting to lock mutex "<<#arg;\
      cerr.flush();\
      pthread_mutex_lock(arg);\
      cerr <<" .. got lock\n";

#undef BIAS_MUTEX_UNLOCK
#define BIAS_MUTEX_UNLOCK(arg)\
      cerr <<__FILE__<<":"<<__LINE__<<" unlocking mutex "<<#arg;\
      cerr.flush();\
      pthread_mutex_unlock(arg);\
      cerr <<" .. unlocked\n";
*/

ZoomFrame::~ZoomFrame()
{}

ZoomFrame::ZoomFrame(wxWindow* parent, const wxString& title)
  : wxFrame(parent, -1, title)
{
  Setup();
  Show();
}

void ZoomFrame::Setup()
{
  int widths[] = { -1, 180 };
  CreateStatusBar(2);
  SetStatusWidths(2, widths);
  
  QuitButton_ = new wxButton(this, MIA_ZoomQuitButton, "Close");

  Window_ = new ZoomImageCanvas(this, GetStatusBar(), 1);
  
  SLScale_ = new wxSlider(this, -1, 10, 1, 25, wxDefaultPosition, 
                          wxSize(-1, -1), wxSL_HORIZONTAL);
  
  wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);

  box->Add(QuitButton_, 0, wxALL | wxGROW, 3);
  box->Add(Window_, 1, wxALL | wxGROW, 3);
  hbox->Add(new wxStaticText( this, -1, _T("Zoom") ), 0, wxALL, 5);
  hbox->Add(SLScale_, 1, wxALL | wxGROW, 5);
  box->Add(hbox, 0, wxGROW, 0);

  box->SetSizeHints(this);
  SetSizer(box);
}

void ZoomFrame::NewImage(BIAS::ImageBase& im, 
                            BIAS::Image<unsigned char>& ucim,
                            const wxString& name)
{
  Window_->NewImage(im, ucim);
  SetStatusText(name, 0);
}

void ZoomFrame::NewCoo(unsigned x, unsigned y)
{ Window_->NewCoo(x, y); }



BEGIN_EVENT_TABLE (ZoomFrame, wxFrame)
  EVT_BUTTON      (MIA_ZoomQuitButton, ZoomFrame::OnQuit)
  EVT_SCROLL      (ZoomFrame::OnZoomFactor)
  EVT_CLOSE       (ZoomFrame::OnClose)
  //EVT_KEY_DOWN    (ZoomFrame::OnKeyPress)
END_EVENT_TABLE();

void ZoomFrame::OnQuit(wxCommandEvent & event)
{ 
  Close(true); 
}

void ZoomFrame::OnClose(wxCloseEvent & event)
{ 
  ((ImageCanvas*)GetParent())->SetZoomed(false);
  Destroy();
}

void ZoomFrame::SetZoom(unsigned z)
{
  BIASASSERT(z>=1);
  BIASASSERT(z<=25);
  SLScale_->SetValue(z);
  Window_->NewZoom(SLScale_->GetValue());
}

void ZoomFrame::OnZoomFactor(wxScrollEvent & event)
{ Window_->NewZoom(SLScale_->GetValue()); }

// void ZoomFrame::OnKeyPress(wxKeyEvent &event)
// {
//   cerr <<"ZoomFrame::OnKeyPress\n";
//   event.Skip();
// }

////////////////////////////////////////////////////////////////////

ZoomImageCanvas::ZoomImageCanvas(wxWindow* parent, wxStatusBar *sbar, 
                                 int sbarid, wxWindowID id, const wxPoint& pos,
                                 const wxSize& size, long style, 
                                 const wxString& name)
  : wxPanel(parent, id, pos, size, style, name)
{
  Bitmap_=NULL;

  wxbitmap_mutex_ = new pthread_mutex_t;
  origim_mutex_ = new pthread_mutex_t;
  pthread_mutex_init(wxbitmap_mutex_, NULL);
  pthread_mutex_init(origim_mutex_, NULL);

  ZoomFactor_=10;
  ZoomX_=0;
  ZoomY_=0;
  SBar_=sbar;
  SBarID_=sbarid;
  mousex_=mousey_=-1;
}

ZoomImageCanvas::~ZoomImageCanvas()
{
  if (wxbitmap_mutex_) delete wxbitmap_mutex_;
  if (origim_mutex_) delete origim_mutex_;
}

void ZoomImageCanvas::GetZoomImage()
{
  #ifdef WIN32
    GetSize(&wwidth_, &wheight_);
    ZoomXPos_=wwidth_/2-ZoomFactor_/2;
    ZoomYPos_=wheight_/2-ZoomFactor_/2;
  #endif //WIN32

  if (zim_.GetWidth()!=(unsigned)wwidth_ || 
      zim_.GetHeight()!=(unsigned)wheight_)
    zim_.Release();
  if (zim_.IsEmpty())
    zim_.Init(wwidth_, wheight_, 3);
  
  zim_.FillImageWithConstValue((unsigned char)0);
  unsigned char **ida = zim_.GetImageDataArray();
  unsigned char **sida= ucim_.GetImageDataArray();
  int xsize=wwidth_/ZoomFactor_/2+1;
  int ysize=wheight_/ZoomFactor_/2+1;
  register unsigned int xstart, ystart;
  register unsigned char res[3];
  unsigned width=ucim_.GetWidth(), height=ucim_.GetHeight();
  int mx, my;
  
  //cout << "wwidth/wheight: "<<wwidth_<<"x"<<wheight_<<endl;
  //cout << "width/height: "<<width<<"x"<<height<<endl;
  //cout << "x "<<(int)ZoomX_-(int)xsize<<" - "<<(int)ZoomX_+(int)xsize<<endl;
  //cout << "y "<<(int)ZoomY_-(int)ysize<<" - "<<(int)ZoomY_+(int)ysize<<endl;
  for (int x=(int)ZoomX_-(int)xsize; x<=(int)ZoomX_+(int)xsize; x++){
    for (int y=(int)ZoomY_-(int)ysize; y<=(int)ZoomY_+(int)ysize; y++){
      if (x<0 || y<0 || x>=(int)width || y>=(int)height){
        //cout <<"out of src range: "<< x << "x" <<y<<endl;
        res[0]=res[1]=res[2]=0;
      } else {
        mx=3*x;
        res[0]=sida[y][mx]; res[1]=sida[y][mx+1]; res[2]=sida[y][mx+2];
      }
      xstart = (x-(int)ZoomX_)*(int)ZoomFactor_+(int)ZoomXPos_;
      ystart = (y-(int)ZoomY_)*(int)ZoomFactor_+(int)ZoomYPos_;
      for (register int i=0; i<(int)ZoomFactor_; i++){
        for (register int j=0; j<(int)ZoomFactor_; j++){
          my=ystart+i; mx=(xstart+j);
          if (mx>=0 && my>=0 && mx<wwidth_ && my<wheight_){
            mx*=3;
            ida[my][mx] = res[0];
            ida[my][mx+1] = res[1];
            ida[my][mx+2] = res[2];
          } 
        }
      }
    }
  }
  BIAS_MUTEX_LOCK(wxbitmap_mutex_);
  wximage_ = new wxImage(zim_.GetWidth(), zim_.GetHeight(), 
                         zim_.GetImageData(), true); 
  BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
  if (mousex_>=0 && mousey_>=0){
    UpdateStatusBar((unsigned) mousex_, (unsigned) mousey_);
  }
}

void ZoomImageCanvas::UpdateStatusBar(unsigned x, unsigned y)
{ 
  if (SBar_){
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;
    BIAS_MUTEX_LOCK(wxbitmap_mutex_);
    BIAS_MUTEX_LOCK(origim_mutex_);
    if (Bitmap_ && ix>=0 && ix<(int)origim_.GetWidth() && 
        iy>=0 && iy<(int)origim_.GetHeight() ){
      BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
      int mix;
      ostringstream os;
      os << "("<<setw(3)<<ix<<", "<<setw(3)<<iy<<") : (";
      switch (origim_.GetChannelCount()){
      case 1:
        if (origim_.GetStorageType()==ImageBase::ST_unsignedchar)
          os << (int)((Image<unsigned char>)origim_).GetImageDataArray()[iy][ix];
        else if (origim_.GetStorageType()==ImageBase::ST_float)
          os << (float)((Image<float>)origim_).GetImageDataArray()[iy][ix];
        break;
      case 3:
        mix=ix*3;
        if (origim_.GetStorageType()==ImageBase::ST_unsignedchar){
          os <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[iy][mix]
             <<", "
             <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[iy][mix+1]
             <<", "
             <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[iy][mix+2];
        } else if (origim_.GetStorageType()==ImageBase::ST_float){
          os <<((Image<float>)origim_).GetImageDataArray()[iy][mix]
             <<", "
             <<((Image<float>)origim_).GetImageDataArray()[iy][mix+1]
             <<", "
             <<((Image<float>)origim_).GetImageDataArray()[iy][mix+2];
        }
        break;
      default:
        BIASERR("???");
        break;
      }
      BIAS_MUTEX_UNLOCK(origim_mutex_);
      os <<")";
      SBar_->SetStatusText(os.str().c_str(), SBarID_);
    } else {
      BIAS_MUTEX_UNLOCK(origim_mutex_);
      BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
      SBar_->SetStatusText("", SBarID_);
    } 
  }
}

void ZoomImageCanvas::NewImage(ImageBase& im, Image<unsigned char>& ucim)
{
  BIAS_MUTEX_LOCK(origim_mutex_);
  origim_ = im;
  BIAS_MUTEX_UNLOCK(origim_mutex_);
  ucim_ = ucim;
  GetZoomImage();
#ifdef WIN32
	this->Refresh();
#else
	wxPaintEvent pe;
	wxPostEvent(this->GetEventHandler(),pe);
#endif
}

void ZoomImageCanvas::NewCoo(unsigned x, unsigned y)
{
  if (ZoomX_!=x || ZoomY_!=y){
    ZoomX_=x;
    ZoomY_=y;
    ZoomXPos_=wwidth_/2-ZoomFactor_/2;
    ZoomYPos_=wheight_/2-ZoomFactor_/2;
    GetZoomImage();
#ifdef WIN32
	this->Refresh();
#else
	wxPaintEvent pe;
	wxPostEvent(this->GetEventHandler(),pe);
#endif
  }
}

void ZoomImageCanvas::NewZoom(int zoom)
{
  if ((unsigned)zoom!=ZoomFactor_){
    ZoomFactor_=zoom;
    GetZoomImage();
#ifdef WIN32
	this->Refresh();
#else
	wxPaintEvent pe;
	wxPostEvent(this->GetEventHandler(),pe);
#endif
  }
}

BEGIN_EVENT_TABLE (ZoomImageCanvas, wxWindow)
  EVT_PAINT       (ZoomImageCanvas::OnPaint)
  EVT_MOTION      (ZoomImageCanvas::OnMouseMove)
  EVT_RIGHT_DOWN  (ZoomImageCanvas::OnRightMouse)
  EVT_LEFT_DOWN   (ZoomImageCanvas::OnLeftMouse)
  EVT_CHAR        (ZoomImageCanvas::OnKey)
  EVT_SIZE        (ZoomImageCanvas::OnResize)
  EVT_LEAVE_WINDOW(ZoomImageCanvas::OnMouseLeave)
END_EVENT_TABLE();

void ZoomImageCanvas::OnRightMouse(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  int x = dc.DeviceToLogicalX( pos.x );
  int y = dc.DeviceToLogicalY( pos.y );
  int ix=(x-(int)(wwidth_>>1))/(int)ZoomFactor_+(int)ZoomX_;
  int iy=(y-(int)(wheight_>>1))/(int)ZoomFactor_+(int)ZoomY_;
  //cerr << "new coo: "<<ix<<", "<<iy; cerr.flush();
  NewCoo(ix, iy);
  //cerr << " .... finished\n";
}

void ZoomImageCanvas::OnMouseLeave(wxMouseEvent &event)
{
  if (SBar_){ SBar_->SetStatusText("", SBarID_); }
  mousex_=mousey_=-1;
}

void ZoomImageCanvas::OnResize(wxSizeEvent &event)
{
  GetSize(&wwidth_, &wheight_);
  ZoomXPos_=wwidth_/2-ZoomFactor_/2;
  ZoomYPos_=wheight_/2-ZoomFactor_/2;
  GetZoomImage();
  event.Skip();
#ifdef WIN32
	this->Refresh();
#else
	wxPaintEvent pe;
	wxPostEvent(this->GetEventHandler(),pe);
#endif
}

void ZoomImageCanvas::OnPaint(wxPaintEvent & event)
{
  //  wxPanel::OnPaint(event);
  
  wxPaintDC dc(this);
  PrepareDC(dc);
  BIAS_MUTEX_LOCK(wxbitmap_mutex_);
  if (wximage_ && wximage_->Ok()){
    if (Bitmap_ != NULL) delete Bitmap_;
    Bitmap_ = new wxBitmap(*wximage_);
  }
  if (Bitmap_ && Bitmap_->Ok() && dc.Ok()) {
    if (ClearDrawingContext_) dc.Clear();
    dc.DrawBitmap(*Bitmap_, 0, 0);
  }
  BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
}

void ZoomImageCanvas::OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  mousex_ = dc.DeviceToLogicalX( pos.x );
  mousey_ = dc.DeviceToLogicalY( pos.y );
  UpdateStatusBar((unsigned) mousex_, (unsigned) mousey_);
}


void ZoomImageCanvas::OnLeftMouse(wxMouseEvent &event)
{
  // pass event through to ImageCanvas which is parent of parent
  if (GetParent()){
    //  if (GetParent() && GetParent()->GetParent()){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned int x = dc.DeviceToLogicalX( pos.x );
    unsigned int y = dc.DeviceToLogicalY( pos.y );
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;

    wxMouseEvent ke=event;
    ke.m_x=ix;
    ke.m_y=iy;
    wxPostEvent(GetParent()->GetParent()->GetEventHandler(), ke);
    //cerr << "passing mouse event at ("<<ix<<", "<<iy
    //     <<") from ZoomImageCanvas to ImageCanvas\n";
  } else {
    BIASERR("no ImageCanvas belonging to ZoomImageCanvas\n");
  }
  event.Skip();
}


void ZoomImageCanvas::OnKey(wxKeyEvent &event) 
{
  // pass event through to ImageCanvas which is parent of parent
  if (GetParent()){
    //  if (GetParent() && GetParent()->GetParent()){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned int x = dc.DeviceToLogicalX( pos.x );
    unsigned int y = dc.DeviceToLogicalY( pos.y );
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;
 
    wxKeyEvent ke=event;
    ke.m_x=ix;
    ke.m_y=iy;
    ke.m_keyCode=event.GetKeyCode();
    wxPostEvent(GetParent()->GetParent()->GetEventHandler(), ke);
    // cerr << "passing KEY event at ("<<ix<<", "<<iy
    //     <<") from ZoomImageCanvas to ImageCanvas\n";
  } else {
    BIASERR("no ImageCanvas belonging to ZoomImageCanvas\n");
  }
  event.Skip();
}

////////////////////////////////////////////////////////////////////////





ImageCanvas::ImageCanvas(wxWindow* parent, wxStatusBar* sbar, int sbarid, 
                         wxWindowID id, const wxPoint& pos, const wxSize& size,
                         long style, const wxString& name) 
  : wxScrolledWindow(parent, id, pos, size, style, name), origim_()
{
  Bitmap_=(wxBitmap *)NULL;
  wximage_=(wxImage *)NULL;
  SBar_=sbar;
  SBarID_=sbarid;

  wxbitmap_mutex_ = new pthread_mutex_t;
  origim_mutex_ = new pthread_mutex_t;
  pthread_mutex_init(wxbitmap_mutex_, NULL);
  pthread_mutex_init(origim_mutex_, NULL);

  Zoomed_=false;
  Title_=name;
  mousex_=mousey_=-1;
  DefaultZoom_=5;
  ClearDrawingContext_=false;
  ShowScaled_ = false;
}

ImageCanvas::~ImageCanvas()
{
  if (wximage_ != NULL) delete wximage_;
  if (Bitmap_ != NULL) delete Bitmap_;

  if (wxbitmap_mutex_) delete wxbitmap_mutex_;
  if (origim_mutex_) delete origim_mutex_;
}

void ImageCanvas::Show(PyramidImage<unsigned char>& pim, string name)
{
  Image<unsigned char> im;
  pim.GetSingleImage(im);
  Show(im, name);
}


void ImageCanvas::Show(BIAS::Image<unsigned char>& im, string name)
{
  if (im.GetColorModel()!=ImageBase::CM_RGB){
    Image<unsigned char> tmpim;
    if (ImageConvert::Convert(im, tmpim, ImageBase::CM_RGB)!=0){
      BIASERR("error converting");
    }
    BIASDOUT(D_IC_IM,"Show(im,name) called Show(im,tmpim,name)");
    Show(im, tmpim, name);
  } else {
    BIASDOUT(D_IC_IM,"Show(im,name) called Show(im,im,name)");
    Show(im, im, name);
  }
}


void ImageCanvas::Show(BIAS::ImageBase& origim, 
                       BIAS::Image<unsigned char>& scaledrgbim, 
                       std::string name)
{
  BIAS_MUTEX_LOCK(origim_mutex_);
  BIASDOUT(D_IC_IM,"Adress of origim: "<<&origim);
  origim_=origim;
  BIAS_MUTEX_UNLOCK(origim_mutex_);
  rgbim_=scaledrgbim;
  BIASDOUT(D_IC_IM,"Adress of rgbim_: "<<&rgbim_);  
  width_=origim.GetWidth();
  height_=origim.GetHeight();
  ImageName_=name;
  if(ShowScaled_) {
    int CanvasWidth, CanvasHeight;
    GetSize(&CanvasWidth,&CanvasHeight);
    SetVirtualSize(CanvasWidth,CanvasHeight );
    SetScrollbars(0, 0, CanvasWidth,CanvasHeight);
  } else {
    SetVirtualSize(width_, height_);
    SetScrollbars(1, 1, width_, height_);
    // workaround: the scrollbars do not dissapear when the size is equal 
    // to the virtual size
    SetSizeHints(-1, -1, width_+SCROLLBAR_WIDTH, height_+SCROLLBAR_WIDTH);
    //SetSizeHints(-1, -1, width_, height_);
  }
  BIAS_MUTEX_LOCK(wxbitmap_mutex_);
  if (wximage_ != NULL) delete wximage_;
  wximage_ =new wxImage(rgbim_.GetWidth(), rgbim_.GetHeight(), 
			rgbim_.GetImageData(), true);
  BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
  if (Zoomed_) {
    //BIAS_MUTEX_LOCK(origim_mutex_);
    MyZoomWin_->NewImage(origim_, rgbim_, ImageName_.c_str());
    //BIAS_MUTEX_UNLOCK(origim_mutex_);
  }
  if (mousex_>=0 && mousey_>=0){
    UpdateStatusBar((unsigned)mousex_, (unsigned)mousey_);
  }
#ifdef WIN32
	this->Refresh();
#else
	wxPaintEvent pe;
	wxPostEvent(this->GetEventHandler(),pe);
#endif
}
 
void ImageCanvas::Zoom(unsigned x, unsigned y)
{
  if (!Zoomed_){
    Zoomed_=true;
    MyZoomWin_=new ZoomFrame(this, Title_);
    //BIAS_MUTEX_LOCK(origim_mutex_);
    MyZoomWin_->NewImage(origim_, rgbim_, ImageName_.c_str());
    //BIAS_MUTEX_UNLOCK(origim_mutex_);
    MyZoomWin_->SetZoom(DefaultZoom_);
  }
  if (ShowScaled_) {
    int CanvasWidth, CanvasHeight;
    GetSize(&CanvasWidth,&CanvasHeight);
    
    MyZoomWin_->NewCoo((int)((float)x * ((float)width_ / (float)CanvasWidth)),
		       (int)((float)y *((float)height_/(float)CanvasHeight)));
  }
  else MyZoomWin_->NewCoo(x, y);
}

void ImageCanvas::UpdateStatusBar(unsigned x, unsigned y)
{
  if (SBar_){
    BIAS_MUTEX_LOCK(wxbitmap_mutex_);
    if (Bitmap_ && (x>0) && (x<width_) && (y>0) & (y<height_)){
      BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
      ostringstream os;
      os << "("<<setw(3)<<x<<", "<<setw(3)<<y<<") : (";
      BIAS_MUTEX_LOCK(origim_mutex_);
      switch (origim_.GetChannelCount()){
      case 1:
        if (origim_.GetStorageType()==ImageBase::ST_unsignedchar)
          os << (int)((Image<unsigned char>)origim_).GetImageDataArray()[y][x];
        else if (origim_.GetStorageType()==ImageBase::ST_float)
          os << (float)((Image<float>)origim_).GetImageDataArray()[y][x];
        break;
      case 3:
        if (origim_.GetStorageType()==ImageBase::ST_unsignedchar){
          os <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[y][x*3]
             <<", "
             <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[y][x*3+1]
             <<", "
             <<(int)
            ((Image<unsigned char>)origim_).GetImageDataArray()[y][x*3+2];
        } else if (origim_.GetStorageType()==ImageBase::ST_float){
	  	    
          os <<((Image<float>)origim_).GetImageDataArray()[y][x*3]
             <<", "
             <<((Image<float>)origim_).GetImageDataArray()[y][x*3+1]
             <<", "
             <<((Image<float>)origim_).GetImageDataArray()[y][x*3+2];
        }
        break;
      default:
        BIASERR("???");
        break;
      }
      BIAS_MUTEX_UNLOCK(origim_mutex_);
      os <<")";
      SBar_->SetStatusText(os.str().c_str(), SBarID_);
    } else {
      SBar_->SetStatusText("", SBarID_);
      BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);
    }
  }
}

bool ImageCanvas::GetMousePos(int& x, int& y)
{
  x= mousex_; y=mousey_;
  return ((mousex_>=0) && (mousey_>=0));
}

///////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE (ImageCanvas, wxScrolledWindow)
  EVT_PAINT       (ImageCanvas::OnPaint)
  EVT_ERASE_BACKGROUND       (ImageCanvas::OnErase)	
  EVT_MOTION      (ImageCanvas::OnMouseMove)
  EVT_RIGHT_DOWN  (ImageCanvas::OnRightMouseButton)
  EVT_LEAVE_WINDOW(ImageCanvas::OnMouseLeave)
  //EVT_SIZE        (ImageCanvas::OnSize)
  //EVT_KEY_DOWN    (ImageCanvas::OnKeyPress)
END_EVENT_TABLE();

void ImageCanvas::OnErase(wxEraseEvent& event){

}


void ImageCanvas::OnPaint(wxPaintEvent & event)
{ 
  wxScrolledWindow::OnPaint(event);
  wxPaintDC dc(this);
  
  PrepareDC(dc);

  int CanvasWidth, CanvasHeight;
  GetSize(&CanvasWidth,&CanvasHeight);


  BIAS_MUTEX_LOCK(wxbitmap_mutex_);
    if (wximage_ && wximage_->Ok()){
      if (Bitmap_ != NULL) delete Bitmap_;
      if (ShowScaled_)  {
	wximage_->Rescale(CanvasWidth,CanvasHeight);
      }
      Bitmap_ = new wxBitmap( *wximage_ );
    }

  if (Bitmap_ && Bitmap_->Ok() && dc.Ok()) {
    // set ClearDrawingContext_ to fals to prevent flickering
	if (ClearDrawingContext_) dc.Clear();
    dc.DrawBitmap( *Bitmap_, 0, 0 );
	
  }
  BIAS_MUTEX_UNLOCK(wxbitmap_mutex_);

  //wxPostEvent(this->GetParent()->GetEventHandler(), event);
}

void ImageCanvas::OnMouseLeave(wxMouseEvent &event)
{ 
  if (SBar_) SBar_->SetStatusText("", SBarID_); 
  mousex_=mousey_=-1;
}

void ImageCanvas::OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  if (GetParent()) GetParent()->PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  mousex_ = dc.DeviceToLogicalX( pos.x );
  mousey_ = dc.DeviceToLogicalY( pos.y );

  if (ShowScaled_) {
    // if we are in scaled mode, correct coordinates by scale !
    int CanvasWidth, CanvasHeight;
    GetSize(&CanvasWidth,&CanvasHeight);
    double h = CanvasHeight, w = CanvasWidth;
    double ow = origim_.GetWidth(), oh = origim_.GetHeight();
    double ratiox = ow/w , ratioy = oh/h;
    mousex_ = (int)rint(mousex_ * ratiox);
    mousey_ = (int)rint(mousey_ * ratioy);
  }
  UpdateStatusBar((unsigned)mousex_, (unsigned)mousey_);
}

void ImageCanvas::OnRightMouseButton(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  unsigned x = (unsigned) dc.DeviceToLogicalX( pos.x );
  unsigned y = (unsigned) dc.DeviceToLogicalY( pos.y );
  Zoom(x, y);
}

void ImageCanvas::OnSize(wxSizeEvent& event)
{
  Fit();
  event.Skip();
}

// void ImageCanvas::OnKeyPress(wxKeyEvent &event)
// {
//   cerr <<"ImageCanvas::OnKeyPress\n";
//   event.Skip();
// }


//////////////////////////////////////////////////////////////////////////

#define SLIDER_DIGITS 1000

enum { ID_Scale=1, ID_Offset};

ScaledImageCanvas::ScaledImageCanvas(wxWindow* parent, wxStatusBar* sbar, 
                                     int sbarid, wxWindowID id, 
                                     const wxPoint& pos, const wxSize& size, 
                                     long style, const wxString& name)
  : wxPanel(parent, id, pos, size, style, name), scaledf_()
{ 
  MiaScWi_ = new ImageCanvas(this, sbar, sbarid, -1, wxDefaultPosition,
                             wxSize(320, 240), wxHSCROLL | wxVSCROLL, name);
  //ScaleSL_ = new wxSlider(this, ID_Scale, 100, 10, 500, wxDefaultPosition, 
  ScaleSL_ = new wxSlider(this, ID_Scale, 100, 0, SLIDER_DIGITS, 
                          wxDefaultPosition, wxSize(-1, -1), wxSL_HORIZONTAL);
  //OffsSL_ = new wxSlider(this, ID_Offset, 0, -100, +100, wxDefaultPosition, 
  OffsSL_ = new wxSlider(this, ID_Offset, 500, 0, SLIDER_DIGITS, 
                         wxDefaultPosition, wxSize(-1, -1), wxSL_HORIZONTAL);
  wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL); 
  wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL); 
  hbox->Add(new wxStaticText( this, -1, _T("Scale") ), 0, wxALL, 3);
  hbox->Add(ScaleSL_, 2, wxALL | wxGROW, 3);
  hbox->Add(new wxStaticText( this, -1, _T("Offset") ), 0, wxALL, 3);
  hbox->Add(OffsSL_, 1, wxALL | wxGROW, 3);
  vbox->Add(MiaScWi_, 1, wxALL | wxGROW, 5);
  vbox->Add(hbox, 0, wxGROW, 0);
  vbox->SetSizeHints(this);
  SetSizer(vbox);
  MinScale_=1.0;
  MaxScale_=50.0;
  MinOffSet_=-100.0;
  MaxOffSet_=100.0;
  p_StatBar=NULL;
  AutoadaptScale_ = true;
}


ScaledImageCanvas::~ScaledImageCanvas()
{
  if (MiaScWi_) delete MiaScWi_;
  if (ScaleSL_) delete ScaleSL_;
  if (OffsSL_) delete OffsSL_;
}


void ScaledImageCanvas::SetStatusBarPtr(wxStatusBar* p_sbar){
    p_StatBar=p_sbar;
}

void ScaledImageCanvas::SetStatusText(const std::string & s){
    if(p_StatBar==NULL){
        wxMessageBox( string("p_StatBar NULL. "+ s).c_str() );
        return;
    }
    BIASASSERT(p_StatBar!=NULL);
    p_StatBar->SetStatusText(s.c_str() );
}


void ScaledImageCanvas::Show(ImageBase& im, string name)
{
  ImageName_=name;
  switch (im.GetStorageType()) {
  case ImageBase::ST_float:
    origim_=im;
    break;
//   case ImageBase::ST_unsignedshortint:
  default:
    ImageConvert::ConvertST(im, origim_, ImageBase::ST_float);
  break;
  }
  scaledf_=(Image<float>)origim_;
  if (AutoadaptScale_) {
    double scale=0, shift=0;
    scaledf_.CalcScaleShift(0.0, 255.0,scale, shift);
    SetOffset(shift);
    SetScale(scale);
  }
   UpdateScaleOffset();
}


void ScaledImageCanvas::Show(PyramidImage<float>& pim, string name)
{
  Image<float> im;
  pim.GetSingleImage(im);
  Show(im, name);
}

int ScaledImageCanvas::SetScale(double scale)
{
  if (scale < MinScale_ || scale > MaxScale_){
    BIASERR("invalid scale");
    return -1;
  }
  int value=(int)rint((scale-MinScale_)*(double)SLIDER_DIGITS/
                      (MaxScale_-MinScale_));
  BIASASSERT(value>=0);
  BIASASSERT(value<=1000);
  ScaleSL_->SetValue(value);
  UpdateScaleOffset();
  return 0;
}

int ScaledImageCanvas::SetOffset(double offset)
{
  if (offset < MinOffSet_ || offset > MaxOffSet_){
    BIASERR("invalid offset");
    return -1;
  }
  int value=(int)rint((offset-MinOffSet_)*(double)SLIDER_DIGITS/
                      (MaxOffSet_-MinOffSet_));
  BIASASSERT(value>=0);
  BIASASSERT(value<=1000);
  OffsSL_->SetValue(value);
  UpdateScaleOffset();
  return 0;
}      

void ScaledImageCanvas::UpdateScaleOffset()
{
  if (!origim_.IsEmpty()){
    scaledf_=(Image<float>)origim_;
    double sc=1.0, ofs=0.0;
    sc=((double)(ScaleSL_->GetValue())*(MaxScale_-MinScale_)/
        (double)SLIDER_DIGITS)+MinScale_;

    ofs=((double)(OffsSL_->GetValue())*(MaxOffSet_-MinOffSet_)/
         (double)SLIDER_DIGITS)+MinOffSet_;

    //cerr << "Scale: "<<sc<<"\tOffset: "<<ofs<<endl;
    stringstream ss;
    ss.str("");
    ss<<"scale "<<sc<<" offset: "<<ofs;
    //if (p_parent!=NULL) p_parent->SetStatusText(ss.str().c_str() );

    scaledf_.ScaleShift(sc, ofs);
    scaledf_.BelowThresholdToValue(0.0, 0.0);
    scaledf_.AboveThresholdToValue(255.0, 255.0);
    
    float **sida=scaledf_.GetImageDataArray();
    float **ida=(float **)origim_.GetImageDataArray();
    unsigned w=origim_.GetWidth(), h=origim_.GetHeight();
    for (unsigned y=0; y<h; y++){
      for (unsigned x=0; x<w; x++){
        if (ida[y][x]==0.0) sida[y][x]=0.0;
      }
    }
    if (ImageConvert::ConvertST(scaledf_, ucim_, 
                                ImageBase::ST_unsignedchar)!=0){
      BIASERR("error converting to unsigned char");
    }
    if (ucim_.GetColorModel()!=ImageBase::CM_RGB){
      ImageConvert::Convert(ucim_, rgbim_, ImageBase::CM_RGB);
    } else {
      rgbim_=ucim_;
    }
    MiaScWi_->Show(origim_, rgbim_, ImageName_);
    
    //BIASASSERT(this->SBar_ != NULL);
    ss.str("");
    ss<<"scale= ";
    ss<<sc;
    ss<<" offset=";
    ss<<ofs;
    //MiaScWi_->SBar_->SetStatusText(ss.str(), 0); // SBarID_ );
//     cout<<"DBG "<<ss.str()<<endl;
     /// \todo: write ofsfet and scale to StatusText instead of cout (JW)    
  }
}

ImageCanvas* ScaledImageCanvas::GetImageCanvas(){
    return MiaScWi_;
}

BIAS::ImageBase & ScaledImageCanvas::GetOrigimg(){
    return origim_;
}


BEGIN_EVENT_TABLE (ScaledImageCanvas, wxPanel)
  EVT_SCROLL(ScaledImageCanvas::OnScaleOffset)
END_EVENT_TABLE();

void ScaledImageCanvas::OnScaleOffset(wxScrollEvent & event)
{
  UpdateScaleOffset();
}
