
#include "ImageCanvasGL.hh"

#include <Base/Image/ImageIO.hh>
#include <Base/Math/Vector2.hh>

using namespace std;
// -----------------------------------------

BEGIN_EVENT_TABLE (ImageCanvasGL, wxGLCanvas)
EVT_PAINT     (ImageCanvasGL::OnPaint)
EVT_SIZE      (ImageCanvasGL::OnResize)
//EVT_MOUSEWHEEL(ImageCanvasGL::OnMouseWheel)
//EVT_MOTION(ImageCanvasGL::OnMotion)
//EVT_RIGHT_DOWN  (ImageCanvasGL::OnRightMouseButton)
//EVT_LEAVE_WINDOW(ImageCanvasGL::OnMouseLeave)
//EVT_KEY_DOWN    (ImageCanvasGL::OnKeyPress)
END_EVENT_TABLE();


ImageCanvasGL::~ImageCanvasGL(){};

// ctors: 
ImageCanvasGL::ImageCanvasGL(wxFrame *parent
							 ,const wxPoint & pos
							 ,const wxSize & size
							 ,long style
							 ,const wxString & name
							 ,int* attribList )
							 : wxGLCanvas( (wxFrame*)parent, -1, pos, size, style, name, attribList )
							 ,_InitWithContextDone(false)
							 ,p_valImg(NULL)
							 ,_oglTexTarget(  (GLenum)GL_TEXTURE_2D )
							 ,_oglTexID( (GLuint) 0)
{
	_Owner = parent;
	Init();
};

void ImageCanvasGL::Init(){	
	if (!_InitWithContextDone) InitWithContext();
};



void ImageCanvasGL::InitWithContext(){
	if (_InitWithContextDone) return; /* already done */

	if (!GetContext()) {
		BIASERR("skipped because no Context available.");
		return; // no context available
	};
	bool ContextAvailable = (NULL!=(char*)glGetString(GL_VENDOR));
	if (!ContextAvailable ){
		BIASERR("skipped because no context available ");
		return;
	};

	/* run init with context once */

	// init glew symbols: 
	int err = glewInit();
	if (GLEW_OK != err) { 
		// problem: glewInit failed, something is seriously wrong 
		BIASERR("could not init GLEW. Error: "<<glewGetErrorString(err));
		exit(-1);
	};

	glGetError(); // dummy to clear all previous errors

	_InitWithContextDone=true;
};

void ImageCanvasGL::InitGLStatus() const {
	CHECK_GL_ERROR;

	// set values for clear:
	glClearColor(0, 0, 0, 0); // rgba [0,1]
	glClearDepth(1.0);
	glClearAccum(0,0,0,0);
	glClearStencil(0);
	glClearIndex(0);

	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE); // no color mask RGBA

	// perform clear operation: 
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT );
	//glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// set gl states (on or off)
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
	glDisable(GL_FOG);
	glDisable(GL_NORMALIZE); // just for glNormal
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
	glDisable(GL_STENCIL_TEST);
	//glEnable(GL_TEXTURE_RECTANGLE_NV); // non-pow2 textures?

	// set compare functions
	glDepthFunc(GL_LEQUAL);
	glAlphaFunc(GL_LEQUAL, (GLclampf)1 );

	// set matrices: 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// set shade model (flat or smooth)
	glShadeModel(GL_SMOOTH);
	CHECK_GL_ERROR;
}


/// return GL state info
void ImageCanvasGL::GetInfoGL(std::ostream & os){
	if (!GetContext()) return; // no context available
	this->SetCurrent(); // get the GL rendering focus

	// get OpenGL info strings into std. strings:
	if ( NULL==(char *) glGetString(GL_VENDOR)) {
		BIASERR("No Context available (NULL ptr). Skipping RenderGL::GetInfo."<<endl<<flush);
		CHECK_GL_ERROR;
		return;
	};

	string strVendor = (char *) glGetString(GL_VENDOR);
	CHECK_GL_ERROR;
	string strRenderer = (char *) glGetString(GL_RENDERER);
	CHECK_GL_ERROR;
	string strVersion = (char *) glGetString(GL_VERSION);
	CHECK_GL_ERROR;
	string strExtensions = (char *) glGetString(GL_EXTENSIONS);
	CHECK_GL_ERROR;

	// replace blank by newline:
	unsigned int index = 0;
	while ((index = strExtensions.find(' ', index)) != string::npos) {
		strExtensions[index++]='\n';
	};
	GLint max_texture_units=0;
	glGetIntegerv(GL_MAX_TEXTURE_UNITS, &max_texture_units);
	CHECK_GL_ERROR;

	//GLint max_texture_image_units;
	//glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS, &max_texture_image_units);  
	GLint max_general_combiners;
	glGetIntegerv(GL_MAX_GENERAL_COMBINERS_NV, &max_general_combiners);
	CHECK_GL_ERROR;
	//GLint max_vertex_units;
	//glGetIntegerv(GL_MAX_VERTEX_UNITS_ARB, &max_vertex_units);
	//CHECK_GL_ERROR;
	//GLint max_vertex_shader_instructions;
	//glGetIntegerv(GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT, &max_vertex_shader_instructions);
	//CHECK_GL_ERROR;

	GLint red_bits;
	glGetIntegerv(GL_RED_BITS, &red_bits);
	GLint green_bits;
	glGetIntegerv(GL_GREEN_BITS, &green_bits);
	GLint blue_bits;
	glGetIntegerv(GL_BLUE_BITS, &blue_bits);
	GLint alpha_bits;
	glGetIntegerv(GL_ALPHA_BITS, &alpha_bits);
	GLfloat alpha_bias;
	glGetFloatv(GL_ALPHA_BIAS, &alpha_bias);
	GLfloat alpha_scale;
	glGetFloatv(GL_ALPHA_SCALE, &alpha_scale);
	GLboolean cull_face;
	glGetBooleanv(GL_CULL_FACE, &cull_face);
	GLfloat depth_scale;
	glGetFloatv(GL_DEPTH_SCALE, &depth_scale);
	GLint depth_bits;
	glGetIntegerv(GL_DEPTH_BITS, &depth_bits);
	GLfloat depth_bias;
	glGetFloatv(GL_DEPTH_BIAS, &depth_bias);
	GLfloat depth_range[2];
	glGetFloatv(GL_DEPTH_BIAS, &depth_range[0]);
	GLfloat depth_clear_value;
	glGetFloatv(GL_DEPTH_CLEAR_VALUE, &depth_clear_value);
	GLboolean fog;
	glGetBooleanv(GL_FOG, &fog);
	GLfloat line_width;
	glGetFloatv(GL_LINE_WIDTH, &line_width);
	GLboolean lighting;
	glGetBooleanv(GL_LIGHTING, &lighting);
	GLint max_modelview_stack_depth;
	glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH, &max_modelview_stack_depth);
	GLint max_projection_stack_depth;
	glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH, &max_projection_stack_depth);
	GLint max_texture_stack_depth;
	glGetIntegerv(GL_MAX_TEXTURE_STACK_DEPTH, &max_texture_stack_depth);
	GLint max_texture_size;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
	GLint max_viewport_dims[2];
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_viewport_dims[0]);
	GLint point_size;
	glGetIntegerv(GL_POINT_SIZE, &point_size);
	GLint point_size_range[2];
	glGetIntegerv(GL_POINT_SIZE_RANGE, &point_size_range[0]);
	GLboolean rgba_mode;
	glGetBooleanv(GL_RGBA_MODE, &rgba_mode);
	CHECK_GL_ERROR;

	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, &viewport[0]);
	GLfloat modelviewMatrix[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, &modelviewMatrix[0]);
	GLfloat projectionMatrix[16];
	glGetFloatv(GL_PROJECTION_MATRIX, &projectionMatrix[0]);
	CHECK_GL_ERROR;

	GLint max_texture_lod_bias;
	glGetIntegerv(GL_MAX_TEXTURE_LOD_BIAS_EXT, &max_texture_lod_bias);
	CHECK_GL_ERROR;

	os << "Getting Info about OpenGL: " << endl;
	os << "GL_VENDOR    = " << strVendor << endl;
	os << "GL_RENDERER  = " << strRenderer << endl;
	os << "GL_VERSION   = " << strVersion << endl;
	os << "GL_EXTENSIONS= " << endl << strExtensions << endl;
	os << endl;

	os<<"GL_RED_BITS= "<<red_bits<<endl;
	os<<"GL_GREEN_BITS= "<<green_bits<<endl;
	os<<"GL_BLUE_BITS= "<<blue_bits<<endl;
	os<<"GL_ALPHA_BITS= "<<alpha_bits<<endl;
	os<<"GL_DEPTH_BITS= "<<depth_bits<<endl;

	//os << "GL_MAX_TEXTURE_UNITS= "<<max_texture_units<<" (detected texture units)" << endl;
	//os << "GL_MAX_TEXTURE_IMAGE_UNITS= " << max_texture_image_units <<endl;
	//os << "GL_MAX_GENERAL_COMBINERS_NV= " << max_general_combiners<<endl;
	//os << "GL_MAX_VERTEX_UNITS = " << max_vertex_units <<" (detected vertex units)" << endl;
	//os << "GL_MAX_VERTEX_SHADER_INSTRUCTIONS= "<<max_vertex_shader_instructions << endl;
	os << "GL_ALPHA_SCALE      = " << alpha_scale << endl;
	os << "GL_ALPHA_BIAS       = " << alpha_bias << endl;
	os << "GL_ALPHA_BITS       = " << alpha_bits << endl;
	os << "GL_CULL_FACE        = " << cull_face << endl;

	os << "GL_DEPTH_SCALE      = " << depth_scale << endl;
	os << "GL_DEPTH_BIAS       = " << depth_bias << endl;
	os << "GL_DEPTH_RANGE      = " << depth_range[0] << " " <<depth_range[1] << endl;
	os << "GL_DEPTH_CLEAR_VALUE= " << depth_clear_value << endl;
	os << "GL_FOG              = " << fog << endl;
	os << "GL_LINE_WIDTH       = " << line_width << endl;
	os << "GL_LIGHTING         = " << lighting << endl;
	os << "GL_MAX_MODELVIEW_STACK_DEPTH = " <<max_modelview_stack_depth << endl;
	os << "GL_MAX_PROJECTION_STACK_DEPTH= " <<max_projection_stack_depth << endl;
	os << "GL_MAX_TEXTURE_LOD_BIAS_EXT  = " << max_texture_lod_bias << endl;
	os << "GL_MAX_TEXTURE_STACK_DEPTH   = " << max_texture_stack_depth<< endl;
	os << "GL_MAX_TEXTURE_SIZE = " << max_texture_size << endl;
	os << "GL_MAX_VIEWPORT_DIMS= " << max_viewport_dims[0] << " " <<  max_viewport_dims[1] << endl;
	os << "GL_POINT_SIZE       = " << point_size << endl;
	os << "GL_POINT_SIZE_RANGE = " << point_size_range[0] << " " <<  point_size_range[1] << endl;
	os << "GL_RGBA_MODE        = " << (int)rgba_mode << endl;
	os << "GL_VIEWPORT         = x="<<viewport[0]<<" y="<<viewport[1]<<" w="<<viewport[2]<<" h="<<viewport[3]<<endl;

	os << "GL_MODELVIEW_MATRIX = (column-major!)"<<endl;
	for (unsigned int row=0; row<4; row++) {
		os<<std::setprecision(10)<<modelviewMatrix[row+ 0]<<" \t";
		os<<std::setprecision(10)<<modelviewMatrix[row+ 4]<<" \t";
		os<<std::setprecision(10)<<modelviewMatrix[row+ 8]<<" \t";
		os<<std::setprecision(10)<<modelviewMatrix[row+12];
		os<<endl;
	};
	os << "GL_PROJECTION_MATRIX = (column-major!) "<<endl;
	for (unsigned int row=0; row<4; row++) {
		os<<std::setprecision(10)<<projectionMatrix[row+ 0]<<" \t";
		os<<std::setprecision(10)<<projectionMatrix[row+ 4]<<" \t";
		os<<std::setprecision(10)<<projectionMatrix[row+ 8]<<" \t";
		os<<std::setprecision(10)<<projectionMatrix[row+12];
		os<<endl;
	};
	CHECK_GL_ERROR;

	// query residence and priority status of all texobj's
	GLint resident=GL_FALSE;
	GLfloat priority=0;
	int foundTextures=0;
	for (unsigned int i=0; i<1000; i++){
		if (glIsTexture(i)) {
			foundTextures++;
			glBindTexture(GL_TEXTURE_2D, i );
			glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_RESIDENT, &resident);
			glGetTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_RESIDENT, &priority);
			os<<"texID="<<i<<" : resident="<<resident<<" priority="<<(float)priority<<endl;
		};
	};
	os<<"found a total of "<<foundTextures<<" textures/IDs"<<endl;
	CHECK_GL_ERROR;
}


void ImageCanvasGL::DisplayFullQuadTexCoord2D(const bool & flipY)
{	
	BIASASSERT(p_valImg!=NULL);

	float width = 1.0f; // pow2 texture dim is normalized to 0..1
	float height=1.0f; 

	if (IsRectangleTexture()){
		// texRect dim is in pixel, e.g. 0..1024
		width  = (float) p_valImg->GetWidth();
		height = (float) p_valImg->GetHeight();
	};

	// 3 2
	// 0 1
	BIAS::Vector2<float> tc0, tc1, tc2, tc3; // 2d RECT texcoords 
	if (!flipY) {
		// OpenGL orientation Y up (=Y is NOT flipped)
		tc0.Set(0,      0 );
		tc1.Set( width, 0 );
		tc2.Set( width, height);
		tc3.Set( 0,     height);
	} else {
		// y is flipped (MIP img orientation)
		tc0.Set(0,      height );
		tc1.Set( width, height );
		tc2.Set( width, 0);
		tc3.Set( 0,     0);
	};

	glColor3f(1, 1, 0); // pink - fallback color
	glBegin(GL_QUADS);
	glTexCoord2fv( &tc0[0] );
	glVertex3f(-1,-1, -1);
	glTexCoord2fv(&tc1[0]);
	glVertex3f( 1,-1, -1);
	glTexCoord2fv(&tc2[0]);
	glVertex3f( 1, 1, -1);
	glTexCoord2fv(&tc3[0]);
	glVertex3f(-1, 1, -1);
	glEnd();
	CHECK_GL_ERROR;
}


void ImageCanvasGL::DisplayImage1x1(const bool & flipY)
{
	if (p_valImg==NULL) return;

	//CALLINFO;
	CHECK_GL_ERROR;
	BIASASSERT(p_valImg != NULL); 
	BIASASSERT(GetWidthNetto()>0);
	BIASASSERT(GetHeightNetto()>0);
	BIASASSERT(glIsTexture( GetTexID()));

	glEnable(GetTexTarget() );
	glBindTexture(GetTexTarget(), GetTexID() );
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	// just for fixed function pipeline:
	CHECK_GL_ERROR;

	DisplayFullQuadTexCoord2D(flipY);

	glDisable(GetTexTarget() );
	CHECK_GL_ERROR;
}

void
ImageCanvasGL::Display(){
	// CALLINFO;
	if (!GetContext()) return; // no context available
	this->SetCurrent(); // get the GL rendering focus
	Init();
	if (!_InitWithContextDone) return;
	// clear old gl errors by nirvana query
	glGetError(); // dummy to clear all previous errors

	InitGLStatus();

	//glPolygonMode(GL_FRONT, _polygonmodeGL ); // modus for onscreen display 

	DisplayImage1x1(true);

	CHECK_GL_ERROR;
	glFlush(); // async.
	this->SwapBuffers(); // glcanvas swap
}

void ImageCanvasGL::Reshape(){
	//COUT("ImageCanvasGL::Reshape not implemented.");
	if (!GetContext()) return; // no context available

	//int sw, sh,  vw, vh;
	//GetSize(&sw, &sh); 
	//GetVirtualSize(&vw, &vh);
	//cout<<"DBG Reshape Size="<<sw<<"x"<<sh<<" VirtualSize "<<vw<<"x"<<vh<<endl;

	//adapt transformation from normalized device coords to pixel screen coords.
	glViewport(0, 0, GetWidth(), GetHeight() );
	// TODO: gluPerspective / glFrustum ???
}


unsigned int ImageCanvasGL::GetWidth() const{
	int w, h;
	GetSize(&w, &h);
	return w;
}

unsigned int ImageCanvasGL::GetHeight() const{
	int w, h;
	GetSize(&w, &h);
	return h;
}

/// @return aspect 'X/Y' = ratio of width being 'bigger' than reference height
float ImageCanvasGL::GetAspectRatio() const {
	return ((float)GetWidth() / (float)GetHeight());
}


BIAS::Vector3<float>
ImageCanvasGL::GetPixelValueRGB(const wxPoint & pos2d) const {
	BIAS::Vector3<float> data;
	glReadPixels((GLint)pos2d.x,
		(GLint)(GetHeight() -pos2d.y),
		(GLsizei)1,  (GLsizei)1,
		GL_RGB,
		GL_FLOAT,
		(GLvoid*)(&data[0]) );
	return data;
}

BIAS::Vector4<float>
ImageCanvasGL::GetPixelValueRGBA(const wxPoint & pos2d) const {
	BIAS::Vector4<float> data;
	glReadPixels((GLint)pos2d.x,
		(GLint)(GetHeight() -pos2d.y),
		(GLsizei)1,  (GLsizei)1,
		GL_RGBA,
		GL_FLOAT,
		(GLvoid*)(&data[0]) );
	return data;
}


bool 
ImageCanvasGL::CopyInData(const wxImage & img){
	bool initImg=true;
	if (p_valImg == NULL) {
		initImg=true;
	} else if ( 
		((int)p_valImg->GetWidth()  == img.GetWidth())
		&& 
		((int)p_valImg->GetHeight() == img.GetHeight()) )
	{
		initImg=false;
	} else {
		initImg=true;
	};

	if (initImg){
		if (p_valImg!=NULL) delete p_valImg;

		p_valImg=new BIAS::ImageBase(img.GetWidth(), img.GetHeight(), 
			3, 
			BIAS::ImageBase::ST_unsignedchar);
	};

	BIASASSERT( (int)p_valImg->GetWidth()  == img.GetWidth()  );
	BIASASSERT( (int)p_valImg->GetHeight() == img.GetHeight() );
    BIASASSERT(img.GetData()!=NULL);
    // copy data area into valImg: 
	p_valImg->CopyIn_NoInit( img.GetData() );
	return true;	
}


bool 
ImageCanvasGL::CopyInData(BIAS::ImageBase & img){
	bool initImg=true;
	if (p_valImg == NULL) {
		initImg=true;
	} else if ( 
		((int)p_valImg->GetWidth()  == img.GetWidth())
		&& 
		((int)p_valImg->GetHeight() == img.GetHeight()) )
	{
		initImg=false;
	} else {
		initImg=true;
	};

	if (initImg){
		if (p_valImg!=NULL) delete p_valImg;

		p_valImg=new BIAS::ImageBase(img.GetWidth(), img.GetHeight(), 
			3, 
			BIAS::ImageBase::ST_unsignedchar);
	};

	BIASASSERT( (int)p_valImg->GetWidth()  == img.GetWidth()  );
	BIASASSERT( (int)p_valImg->GetHeight() == img.GetHeight() );
    BIASASSERT(img.GetImageData()!=NULL);
    // copy data area into valImg: 
	p_valImg->CopyIn_NoInit( img.GetImageData() );
	return true;	
}




bool 
ImageCanvasGL::SetValueImage(const wxImage & img){
	// copy image into protected container image
	CopyInData( img );
	BIASASSERT(p_valImg!=NULL);
	// (re-init) GL tex because the size may have changed 
	InitGLTexture();
	// copy Data into GL tex obj
	CopyAndBind();
	return true;
}

bool 
ImageCanvasGL::SetValueImage(BIAS::ImageBase & img){
	// copy image into protected container image
	CopyInData( img );
	BIASASSERT(p_valImg!=NULL);
	// (re-init) GL tex because the size may have changed 
	InitGLTexture();
	// copy Data into GL tex obj
	CopyAndBind();
	return true;
}


void 
ImageCanvasGL::DeleteGLTexture() {
	glEnable( _oglTexTarget );
	CHECK_GL_ERROR;
	// free gl texture and ID:
	glDeleteTextures(1, (GLuint*)(&_oglTexID) );
	CHECK_GL_ERROR;
	glDisable( _oglTexTarget );
	CHECK_GL_ERROR;
	_oglTexID=0;
}


void ImageCanvasGL::InitGLTexture() {
	DeleteGLTexture(); // release old texture (if there is one)

	glEnable( _oglTexTarget );
	CHECK_GL_ERROR;

	// generate unique, free texture object id's:
	glGenTextures(1, &_oglTexID);
	glBindTexture(_oglTexTarget, _oglTexID );
	CHECK_GL_ERROR;

	glPixelStorei(GL_PACK_ALIGNMENT, 1);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexParameteri(_oglTexTarget, GL_TEXTURE_WRAP_S, GL_CLAMP); // border handling - repeat or clamp?
	glTexParameteri(_oglTexTarget, GL_TEXTURE_WRAP_T, GL_CLAMP);

	if (IsRectangleTexture()){
		// no mipmapping for rectangle non-power of two texture
		glTexParameteri(_oglTexTarget, GL_TEXTURE_MIN_FILTER,  GL_NEAREST);// minification
		glTexParameteri(_oglTexTarget, GL_TEXTURE_MAG_FILTER,  GL_NEAREST);// magnification
	} else {
		glTexParameteri(_oglTexTarget, GL_GENERATE_MIPMAP, GL_FALSE); // mipmapping ?
		glTexParameteri(_oglTexTarget, GL_TEXTURE_MIN_FILTER,  GL_NEAREST);// minification
		glTexParameteri(_oglTexTarget, GL_TEXTURE_MAG_FILTER,  GL_NEAREST);// magnification
		// selection params for mipmapping/level of detail
		glTexParameteri(_oglTexTarget, GL_TEXTURE_BASE_LEVEL, 0); 
		glTexParameteri(_oglTexTarget, GL_TEXTURE_MAX_LEVEL, 10);
		glTexParameterf(_oglTexTarget, GL_TEXTURE_LOD_BIAS, 0.0);
	};
	CHECK_GL_ERROR;

	BIASASSERT(GetWidthNetto()>0);
	BIASASSERT(GetHeightNetto()>0);
	//color RGB888 interleaved format: RGB RGB RGB
	glTexImage2D( _oglTexTarget , // target
		0, // level
		GL_RGB, // internalformat (in graphics card)
		GetWidthNetto(), // width TODO FIXME use brutto size here (for pow2 size)
		GetHeightNetto(), // height
		0, // border
		GL_RGB, // format of src pixel data
		GL_UNSIGNED_BYTE, // type
		NULL ); // datptr. NULL means just allocate (without copy)

	CHECK_GL_ERROR;
	BIASASSERT( GL_TRUE == glIsTexture( _oglTexID ) ); // success ?
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); // replace, no light (often forgotten...)
	glDisable( _oglTexTarget );

	CHECK_GL_ERROR;
}


unsigned int ImageCanvasGL::GetWidthNetto() const {
	if (p_valImg!=NULL) return p_valImg->GetWidth();
	return 0;
}

unsigned int ImageCanvasGL::GetHeightNetto() const {
	if (p_valImg!=NULL) return p_valImg->GetHeight();
	return 0;
}



void ImageCanvasGL::CopyAndBind(){
	// CALLINFO;
	glEnable(GetTexTarget());
	glBindTexture(GetTexTarget(), GetTexID() );
	CHECK_GL_ERROR;

	BIASASSERT(p_valImg!=NULL);
	BIASASSERT(p_valImg->GetImageData()!=NULL);
	BIASASSERT(p_valImg->GetChannelCount()==3);
	BIASASSERT(p_valImg->GetDepth()==1); // 1Byte per Pixel (for now) TODO: half, float formats using texRECT 

	// image was read form disk and is in memory, now.
	glTexSubImage2D( GetTexTarget(),
		0, // level
		0, // xoffset
		0, // yoffset
		GetWidthNetto(),
		GetHeightNetto(),
		GL_RGB, // format of src pixel data 
		GL_UNSIGNED_BYTE, // type
		p_valImg->GetImageData() ); // ptr to src data 
	BIASASSERT( GL_TRUE == glIsTexture(GetTexID()) );// success 
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); // replace, no light (often forgotten...)

	// disable texturing (again).
	// draw routines should enable texturing if required.
	glDisable( _oglTexTarget );
	CHECK_GL_ERROR;
}

void ImageCanvasGL::SaveFile(const std::string & filename){
	COUT("writing DBG image "<<filename);
	//BIAS::ImageIO::ExportMagickPP("out_DEBUG_p_valImg.jpg", *p_valImg );
	BIAS::ImageIO::ExportImage(filename, *p_valImg, BIAS::ImageIO::FF_jpg );
}



void ImageCanvasGL::OnPaint( wxPaintEvent &WXUNUSED(event) ) {
	// This is a dummy, to avoid an endless succession of paint messages.
	// OnPaint handlers must always create a wxPaintDC.
	wxPaintDC dc(this);
	Display();
}

void ImageCanvasGL::OnResize( wxPaintEvent &WXUNUSED(event) ) {
	Reshape();
}

