#ifndef __ImageCanvasGL_hh__
#define __ImageCanvasGL_hh__


/** \author Jan Woetzel 01/2003 */

// must be first
#include "glstuff.h"

// STD
#include <vector>
#include <iostream>
#include <iomanip>

// BIAS
#include <bias_config.h>
#include <Base/Debug/Error.hh>
#include <Base/Debug/DebugSimple.hh>


// HERE IS THE PROBLEM FIXME
//#include <Base/Image/Image.hh>
#include <Base/Image/ImageBase.hh>


#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector4.hh>


// WX
#include <wx/wx.h>
#include <wx/image.h>


/** \brief 2D image display canvas using OpenGL rendering internally. 
\author Jan Woetzel */
class ImageCanvasGL : public wxGLCanvas 
{	
public:
	~ImageCanvasGL(); // destructor

	// ctors:
	ImageCanvasGL(wxFrame *parent,
		const wxPoint & pos,
		const wxSize & size,
		long style=0,
		const wxString & name="OpenGLCanvas",
		int* attribList = 0);

	void Init();
	void InitNoContext();
	void InitWithContext();	
	void InitGLStatus() const;

	/// return GL state info
	void GetInfoGL(std::ostream & os);

	void Display();
	void Reshape();
	unsigned int GetWidth() const;
	unsigned int GetHeight() const;
	float GetAspectRatio() const;
	BIAS::Vector3<float> GetPixelValueRGB(const wxPoint & pos2d) const;
	BIAS::Vector4<float> GetPixelValueRGBA(const wxPoint & pos2d) const;

	//bool SetValueImage(const BIAS::ImageBase & ValueImage);
	bool SetValueImage(const wxImage & img);

    bool SetValueImage(BIAS::ImageBase & img);

	// copies the data area of img into p_valImg
	bool CopyInData(const wxImage & img);

    bool CopyInData(BIAS::ImageBase & img);

	// utilities: 
	bool IsRectangleTexture() const { return (_oglTexTarget==GL_TEXTURE_RECTANGLE_NV); };
	bool IsPow2Texture() const { return (_oglTexTarget==GL_TEXTURE_2D); };

	// original widthwithgout padding
	unsigned int GetWidthNetto() const;

	// original height withgout padding
	unsigned int GetHeightNetto() const;

	// @return OGL texture target, either: pow2=GL_TEXTURE_2D or rectangle=GL_TEXTURE_RECTANGLE_NV
	GLenum GetTexTarget() const {return _oglTexTarget;};
	GLenum TexTarget() const {return TexTarget();};

	// @return OpenGL texture ID
	GLuint GetTexID() const { return _oglTexID; };
	GLuint TexID() const { return GetTexID(); };

	// create gl tex obj ID and set params
	void InitGLTexture();

	// copy data into Gl and bind it to tex obj
	void CopyAndBind();

	// free GL texture and texture ID
	void DeleteGLTexture();

	void SaveFile(const std::string & filename);

	// display GL fullscrenn one-quad image
	void DisplayImage1x1(const bool & flipY=false);

	// display GL quad with assigned texcoords 
	void DisplayFullQuadTexCoord2D(const bool & flipY=false);

	void OnPaint( wxPaintEvent &event );
	void OnResize( wxPaintEvent &WXUNUSED(event) );
	//void OnMouseWheel( wxMouseEvent &event );
	//void OnMotion( wxMouseEvent &event )

    BIAS::ImageBase * GetValImgPtr(){ return p_valImg; };


protected:
	wxFrame *_Owner; // the parent to allow writing to status bar etc.
	bool _InitNoContextDone;
	bool _InitWithContextDone;

	wxPoint _pos; // mouse position

	// image containing teh original values
	// to be used for displaying value under mouse position
	BIAS::ImageBase * p_valImg;

	// OpenGL texture for display: 
	GLenum _oglTexTarget; // GL_TEXTURE_2D or GL_TEXTURE_RECTANGLE_NV
	GLuint _oglTexID; // unique OpengL texture id

	DECLARE_EVENT_TABLE();
}; // ImageCanvasGL

#endif
