#include "Contour.hh"
  
// std includes
#include <limits>

using namespace std;
using namespace BIAS;

void Contour::Init_(BIAS::Image<unsigned char> &img) 
{
  img_w = img.GetWidth();
  img_h = img.GetHeight();    
  data_  = img.GetImageDataArray();

  // draw v lines
  const int offset = img_w-1;
  for (int i = 0; i < img_h; i++) {
    data_[i][0]      = background_;
    data_[i][offset] = background_;
  }
  // draw h lines
  for (int n = 0; n < img_h; n += (img_h-1)) {
    unsigned char *line = data_[n];
    for (int i = 0; i < img_w; i++) 
      line[i] = background_;     
  }
}


void Contour::FindFirst_(int &pos_x, int &pos_y) {
  for (int y = 0; y < img_h; y++) 
    for (int x = 0; x < img_w; x++) 
      if ( IsFg(x,y) ) {
        pos_x = x;
        pos_y = y;	  
        return;
      }
}


int Contour::Process(BIAS::Image<unsigned char> &img)
{
  contPixels_.clear();
  directionList_.clear();

  Init_(img);

  // find first foreground pixel location
  int first_x = -1;
  int first_y = -1;
  FindFirst_( first_x, first_y );
  if ( first_x < 0 || first_y < 0 )
    return -1;

  // create 
  int x = first_x;
  int y = first_y;
  int d = 0;

  do {
    contPixels_.push_back ( BIAS::Vector2<int>( x,y ) );
    d -= 3;
    while (!IsFg(x,y,d)) {
      d++;
    }
    d = (d+16)%8;
    
    directionList_.push_back ( d );

    switch ( d ) {
    case 0: x += 1; y += 0; break;
    case 1: x += 1; y += 1; break;
    case 2: x += 0; y += 1; break;
    case 3: x -= 1; y += 1; break;
    case 4: x -= 1; y += 0; break;
    case 5: x -= 1; y -= 1; break;
    case 6: x += 0; y -= 1; break;
    case 7: x += 1; y -= 1; break;      
    }
  } while ( x != first_x  ||  y != first_y );  
  return 0;
}

int Contour::Draw(BIAS::Image<unsigned char> &img, 
                unsigned char greyValue)
{
  for (unsigned int i=0;i<contPixels_.size();i++) {
#ifdef BIAS_DEBUG
    if ( (contPixels_[i][1]<0) || contPixels_[i][1]>=(int)img.GetHeight() ||
         (contPixels_[i][0]<0) || contPixels_[i][0]>=(int)img.GetWidth() ) {
      BIASERR("contour does not fit into image");
      return -1;
    }
#endif
    img.GetImageDataArray()[contPixels_[i][1]][contPixels_[i][0]]=greyValue;
  }
  return 0;
}
