#include <Image/CameraParam.hh>
#include <Image/MatlabTBUndistortion.hh>
#include <Base/Image/ImageIO.hh>
#include <iostream>

using namespace BIAS;
using namespace std;

int main (int argc, char* argv[])
{
#ifdef BIAS_HAVE_XML2
  CameraParam cp;
  MatlabTBUndistortion mtu;
  Image<unsigned char> src, dst, dst2;
  // double xDst, yDst;

  cp.ReadCameraData("DaimlerCameraLeft.xml");
  if (cp.IsInited() && cp.IsPerspective()) {
    cout << "DaimlerCameraLeft.xml" << " used for initialization." << endl;
    mtu.Init(cp);
  } else {
    cout << "Error: could not read camera data file or no perspective lens!" 
         << endl;
    abort();
  }
  
  // source image is distorted
  if (ImageIO::Load("DaimlerImage.pgm", src)!=0){
    BIASERR("error loading image ");
    return -1;
  }
  
  // undistort distorted image -> loop over undistorted image and distort
  mtu.SetupProjectionLUT(src.GetWidth(), src.GetHeight(),
                     src.GetChannelCount(),
                     src.GetWidth(), src.GetHeight());
  mtu.UndistortFast(src,dst);
  //mtu.UndistortImage(src,dst);

  ImageIO::Write("UndistortedImage.pgm", dst);
  cout << " created: "<<"UndistortedImage.pgm"<<endl;

  // redistort image again -> loop over distorted image
  mtu.DistortImage(dst,dst2);

  ImageIO::Write("RedistortedImage.pgm", dst2);
  cout << " created: "<<"RedistortedImage.pgm"<<endl;
#endif

  return 0;  
}
