/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <iomanip>

#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <Matcher2D/RegionMatcher.hh>

using namespace BIAS;
using namespace std;

int demo(unsigned int size) {
 Image<unsigned char> im(size, size, 3);
  unsigned int chess_field_size=size/8;
  unsigned int x=0, y=0, c=0, r=0;
  unsigned char **ida=im.GetImageDataArray();
  unsigned char red=89,blue=127;

  for (x=0;x<im.GetWidth(); x+= chess_field_size) {
    //    color = (color==0) ? (unsigned char)255 : (unsigned char)0;
    if (red==127) red=89; else red=127;
    if (blue==127) blue=25; else blue=127;    
    for (y=0;y<im.GetHeight(); y+= chess_field_size) {
     if (red==127) red=89; else red=127;
     if (blue==127) blue=25; else blue=127; 
      //color = (color==0) ? (unsigned char)255 : (unsigned char)0;
      for (r=0; r<chess_field_size; r++)
        for (c=0; c<chess_field_size; c++){
          ida[y+r][(x+c)*3 + 0]=red;
          ida[y+r][(x+c)*3 + 1]=255;
          ida[y+r][(x+c)*3 + 2]=blue;
        }      
    }
  }
  
  // grey level ncc
  Image<unsigned char> imGrey(size,size,1);
  ImageConvert::Convert(im,imGrey,ImageBase::CM_Grey);
  int n=1;
  double result=0;
  BIAS::RegionMatcher matcher;
  //matcher.SetDebugLevel(D_REGION_MATCHER_COLORNCC);
  unsigned int p1[2],p2[2];
  p1[0]=chess_field_size; p1[1]= chess_field_size;
  for (x=chess_field_size-n;x<=chess_field_size+n;x++) {
    for (y=chess_field_size-n;y<=chess_field_size+n;y++) {
      p2[0]=x; p2[1]=y;
      matcher.NCC(p1, p2,imGrey.GetImageDataArray(),imGrey.GetImageDataArray(),
                  3, result);
      cerr<<"["<<p2[0]<<","<<p2[1]<<"]:"<<setprecision(2)<<result<<" ";
    } 
    cerr<<endl;
  }
  cerr<<endl;
  Image<unsigned char> imHSV(size,size,3);
  ImageConvert::Convert(im,imHSV,ImageBase::CM_hsL);
  for (x=chess_field_size-n;x<=chess_field_size+n;x++) {
    for (y=chess_field_size-n;y<=chess_field_size+n;y++) {
      p2[0]=x; p2[1]=y;
      matcher.CNCC(p1, p2,imHSV.GetImageDataArray(),
                       imHSV.GetImageDataArray(),
                       3, result);
                 
      cerr<<"["<<p2[0]<<","<<p2[1]<<"]:"<<setprecision(2)<<result<<" ";
    } 
    cerr<<endl;
  }
  if (ImageIO::Write("chessboard", im)!=0){
    BIASERR("error writing image ");
    return -2;
  }
 if (ImageIO::Write("chessboardGrey", imGrey)!=0){
    BIASERR("error writing image ");
    return -2;
  }
  if (ImageIO::ExportImage("chessboard", im, ImageIO::FF_ppm)!=0){
    BIASERR("error exporting image ");
    return -2;
  }
  return 0;
}

int main(int argc, char *argv[])
{
  unsigned int size=800;
  if (argc<3) 
    exit(demo(size));

  Image<unsigned char> im1,im2;
  if (ImageIO::Load(argv[1],im1)!=0) {
    BIASERR("error loading image "<<argv[1]);
    exit (-1);
  }
  if (ImageIO::Load(argv[2],im2)!=0) {
    BIASERR("error loading image "<<argv[2]);
    exit (-1);
  }
  
                                   
  Image<unsigned char> imGrey1, imHSV1(im1.GetWidth(),im1.GetHeight(),3),
    imGrey2, imHSV2(im2.GetWidth(),im2.GetHeight(),3), imRGB1(im1),imRGB2(im2);
  ImageConvert::Convert(im1,imRGB1,ImageBase::CM_RGB);
  ImageConvert::Convert(im2,imRGB2,ImageBase::CM_RGB);
  ImageConvert::Convert(imRGB1,imGrey1,ImageBase::CM_Grey);
  ImageConvert::Convert(imRGB2,imGrey2,ImageBase::CM_Grey);
  ImageConvert::Convert(imRGB1,imHSV1,ImageBase::CM_HSL);  
  ImageConvert::Convert(imRGB2,imHSV2,ImageBase::CM_HSL);  

  Image<unsigned char> imgGreyRes(imGrey1),imgColorRes(imGrey1);

  // copmute NCC for the whole image (except border)
  BIAS::RegionMatcher matcher;
  //matcher.SetDebugLevel(D_REGION_MATCHER_COLORNCC);
  unsigned int winSizeNCC=3;
  unsigned char **idaRes=imgColorRes.GetImageDataArray();
  unsigned int p1[2],p2[2],x,y;//,r,c;
  double result=0;

  //  x,y:338,68 
 //   p1[0]=p2[0]=250; p1[1]=p2[1]=191;
  

//    matcher.CNCC(p1, p2,imHSV1.GetImageDataArray(),
//                     imHSV2.GetImageDataArray(),
//                     winSizeNCC, result);
//    cerr<<"result:"<<result<<endl;
//    exit(0);
  for (y=winSizeNCC;y<imHSV1.GetHeight()-winSizeNCC-1; y++) {
    for (x=winSizeNCC;x<imHSV1.GetWidth()-winSizeNCC-1; x++) {
      p1[0]=p2[0]=x; p1[1]=p2[1]=y;
      matcher.ColorNCC(p1, p2,imHSV1.GetImageDataArray(),
                       imHSV2.GetImageDataArray(),
                       winSizeNCC, result);
      if (fabs(result)>1) {
        cerr<<"x,y:"<<x<<","<<y<<" result:"<<result<<endl;
      }     
      if (result<0) result=0; 
      idaRes[y][x]=(unsigned char)(result*255.0);
    }
  }

  if (ImageIO::Write("imgColorRes",imgColorRes)!=0){
    BIASERR("error writing image imgColorRes");
    return -2;
  }

  if (ImageIO::Write("imGrey1",imGrey1)!=0){
    BIASERR("error writing image imgColorRes");
    return -2;
  }

  winSizeNCC=3;
  idaRes=imgGreyRes.GetImageDataArray();
  for (x=winSizeNCC;x<imHSV1.GetWidth()-winSizeNCC; x++) {
    for (y=winSizeNCC;y<imHSV1.GetHeight()-winSizeNCC; y++) {
      p1[0]=p2[0]=x; p1[1]=p2[1]=y;
      matcher.NCC(p1, p2,imGrey1.GetImageDataArray(),
                       imGrey2.GetImageDataArray(),
                       winSizeNCC, result);
      if (result<0) result=0;
      idaRes[y][x]=(unsigned char)(result*255.0);
    }
  }
  if (ImageIO::Write("imgGreyRes",imgGreyRes )!=0){
    BIASERR("error writing image imgColorRes");
    return -2;
  }
  return 0;
}
