## 
## ctest -S script
## used by Dart/CMake for nightly build setup.
## 
## For additional information see the howto on
## http://public.kitware.com/Wiki/CMake_Scripting_Of_CTest
## 
## or contact the author: 
##
## Jan Woetzel 02/2005 
##


## the cvs module name that should be checked out 
SET (CTEST_MODULE_NAME     "BIAS")

## IDS to identify this build type
SET (CTEST_MODULE_ID       "1012")
SET (CTEST_MODULE_OS       "Win32")
# SET (CTEST_MODULE_COMPILER "nmake71")
SET (CTEST_MODULE_COMPILER "vs71")


## set the build type for picking compiler flags
## argument is accepted as comma separated, e.g.: ctest -V -S bias.cmake,Release
IF (CTEST_SCRIPT_ARG)
  SET (CTEST_BUILD_TYPE "${CTEST_SCRIPT_ARG}")
ELSE (CTEST_SCRIPT_ARG)
  SET (CTEST_BUILD_TYPE "Debug") 
ENDIF (CTEST_SCRIPT_ARG)
MESSAGE(SEND_ERROR "DBG xxxxxxxxxxxxxxx CTEST_BUILD_TYPE=${CTEST_BUILD_TYPE} xxxxxxxxxxxxxxx")


## the root dir for source, build and our additional CMakeModules 
# SET(DASHBOARD_DIR "D:/home/nightlybuild/DartDashboards")
SET(DASHBOARD_DIR "$ENV{HOME}/DartDashboards/DartDashboards")


## Experimental Nightly Continuous, ...
# SET (MODEL "Experimental")
SET (MODEL "Nightly")


## user name
IF (WIN32)
  SET(USER "$ENV{USERNAME}")
ELSE(WIN32)
  SET(USER "$ENV{USER}")
ENDIF(WIN32)
## username OK? 
IF(NOT USER)
  SET(USER "nightlybuild")
  MESSAGE(SEND_ERROR "XXXXXXXXXX Error: USER could not be determined. defaulting to USER=${USER} XXXXXXXXXX")
ENDIF(NOT USER)


## code src directory
SET(CTEST_SOURCE_DIRECTORY 
  "${DASHBOARD_DIR}/source/${CTEST_MODULE_NAME}" )

## compilation directory 
SET(CTEST_BINARY_DIRECTORY 
  "${DASHBOARD_DIR}/build/${CTEST_MODULE_NAME}${CTEST_MODULE_ID}" )

## our additional cmake modules and find scripts
SET (CMAKE_MODULE_PATH
  "${DASHBOARD_DIR}/source/CMakeModules")

## which binaries to use for running the dashboard 
IF(WIN32)
  SET (CTEST_COMMAND       "$ENV{ProgramFiles}/CMake20/bin/ctest.exe")
  SET (CTEST_CMAKE_COMMAND "$ENV{ProgramFiles}/CMake20/bin/cmake.exe")
  SET (CTEST_CVS_COMMAND   "$ENV{ProgramFiles}/TortoiseCVS/cvs.exe")
ELSE(WIN32)
  SET (CTEST_COMMAND       "/usr/net/bin/ctest")
  SET (CTEST_CMAKE_COMMAND "/usr/net/bin/cmake")
  SET (CTEST_CVS_COMMAND   "/usr/bin/cvs")
ENDIF(WIN32)


##
## ctest command to run
## Please mind escaping all backslash and quotes
SET(CTEST_COMMAND "${CTEST_COMMAND} -D ${MODEL} -V -A \"${CTEST_BINARY_DIRECTORY}/bias_config.h\\;${CTEST_SCRIPT_DIRECTORY}/${CTEST_SCRIPT_NAME}\"  ")


##
## cvs command used for check out
##
SET (CTEST_CVS_CHECKOUT 
  "${CTEST_CVS_COMMAND} -d :pserver:${USER}@leine:65010/home/cvs/bias co -d \"${CTEST_SOURCE_DIRECTORY}\" ${CTEST_MODULE_NAME}" )


#####################################################################
## Specify the build options, here
## You could use cmake -D with an initial cache file instead, too.
#####################################################################

## ctests should wipe the binary tree before running
SET (CTEST_START_WITH_EMPTY_BINARY_DIRECTORY TRUE)

## this is the initial cache to use for the binary tree.
## Be careful to escape any quotes inside of this string
SET (CTEST_INITIAL_CACHE "
// Visual Studio is default - nothing to do here.
//Command used to build entire project from the command line.
//MAKECOMMAND:STRING=nmake -i 
//MAKECOMMAND:STRING=\"C:/Programme/Microsoft Visual Studio .NET 2003/Common7/IDE/devenv.com\" BIAS.sln /build Debug /project ALL_BUILD
//CMAKE_MAKE_PROGRAM:FILEPATH=nmake
//CMAKE_MAKE_PROGRAM:FILEPATH=\"C:/Programme/Microsoft Visual Studio .NET 2003/Common7/IDE/devenv.com\"

//CMAKE_GENERATOR:INTERNAL=NMake Makefiles
CMAKE_GENERATOR:INTERNAL=Visual Studio 7 .NET 2003

CVSCOMMAND:FILEPATH=${CTEST_CVS_COMMAND}
// identifier for the build, a short name. You cann append additional short info here
BUILDNAME:STRING=${CTEST_MODULE_OS}_${CTEST_MODULE_COMPILER}_${CTEST_MODULE_NAME}_${CTEST_MODULE_ID}_${CTEST_BUILD_TYPE}_${USER}_mostON_static
// build options handed down to cmake
CMAKE_BUILD_TYPE:STRING=${CTEST_BUILD_TYPE}
CMAKE_VERBOSE_MAKEFILE:BOOL=ON
BUILD_TESTS:BOOL=ON
BUILD_TESTING:BOOL=ON
// our own project specific build options
BUILD_DOXYGEN:BOOL=OFF
BUILD_SHARED_LIBS:BOOL=OFF
BUILD_EXAMPLES:BOOL=ON
BUILD_IMAGE_CHAR:BOOL=ON
BUILD_IMAGE_INT:BOOL=ON
BUILD_IMAGE_SHORT:BOOL=ON
BUILD_IMAGE_UINT:BOOL=ON
BUILD_IMAGE_USHORT:BOOL=ON
BUILD_IMAGE_DOUBLE:BOOL=ON
// third party libs
USE_DAIMLERCHRYSLER:BOOL=OFF
USE_FFTW3:BOOL=OFF
USE_GSL:BOOL=OFF
USE_GTK:BOOL=OFF
USE_GTK2:BOOL=OFF
USE_IMAGEMAGICKLIB:BOOL=ON
USE_IMLIB:BOOL=OFF
USE_OPENCV:BOOL=OFF
USE_OPENGL:BOOL=ON
USE_WXWINDOWS:BOOL=ON
USE_XML2:BOOL=ON
")


## memory leak checking, e.g. with valgrind on Linux
## MEMORYCHECK_COMMAND:FILEPATH=/usr/bin/valgrind


## set any OS envionment variables here
## we can avoid inheritance from user settings with this approach
## multiple settings are allowed, eg. PATH, CC, CXX etc.
SET (CTEST_ENVIRONMENT 
 "CMAKE_MODULE_PATH=${CMAKE_MODULE_PATH}"
)


##
## run an extra cvs update first on up to ten additional directories
## CMakeModule is requird due to changes of FindXXX.cmake files.
SET (CTEST_EXTRA_UPDATES_1 "${CMAKE_MODULE_PATH}" "-dAP")
## DartConfig is required due to possible change of CVS checkout time
SET (CTEST_EXTRA_UPDATES_2 "${CTEST_SOURCE_DIRECTORY}" " -dAP DartConfig.cmake")
SET (CTEST_EXTRA_UPDATES_3 "${CTEST_SOURCE_DIRECTORY}" " -dAP CMakeLists.txt")

