/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Base/Image/Image.hh>
#include <Filter/Mean.hh>
  //#include <Filter/RankValue.hh>
#include <Base/Image/ImageIO.hh>

#ifndef  WIN32
#include <unistd.h>
#else  //WIN32
#include "Base/Common/getopt_W32.h"
#endif //WIN32

using namespace std;
using namespace BIAS;


enum filtertype {none, mean, median};


void usage()
{
  cout <<"Usage: biasfilter [options] infile outfile"<<endl
       <<" Options: "<<endl
       <<"-r radius  : size of the filter kernel: size = 2 * radius + 1"<<endl
       <<"-a         : use mean filter"<<endl
       <<"-b         : use median filter"<<endl
       <<"-v         : verbose"<<endl
       <<"-t thresh  : median: ignore x<=thresh"<<endl;
}

/** @class biasfilter
    @ingroup g_tools
    @brief tool to filter an image with Mean or Median,...
    @author evers */
int main(int argc, char *argv[])
{

  Mean<unsigned char, unsigned char> meanfilterUC;
  Mean<float, float> meanfilterFLT;
  //  RankValue rankvaluefilter;
  filtertype UseFilter = none;
  int rad = -1;
  //int c;
  int c=1;
  float threshold = -FLT_MAX;
  bool verbose = false;

  while(c >0) {
    c = getopt(argc,argv,"r:abt:v");
    if (c>0){
      switch(c){
      case 'a': UseFilter = mean;
	break;
      case 'b': UseFilter = median;
	break;
      case 'r': rad = atoi(optarg);
	break;
      case 't': threshold = float(atof(optarg));
	break;
      case 'v': verbose = true;
	break;
	//     case 'h' : 
      default:
	usage();
	exit(1);
      }
    }
  }

  if (argc-optind <2 ) {
    cout <<"==> To few arguments."<<endl;
    usage();
    exit(1);
  }

  if (UseFilter == none ) {
    cout <<"==> No filtertype specified."<<endl;
    usage();
    exit(1);
  }

  if (verbose){
    switch (UseFilter) {
    case median: cout<<"Median filter";
      break; 
    case mean: cout<<"Median filter";
      break; 
    default: ;
    }
    cout <<" size "<<2*rad+1<<"x"<<2*rad+1;
    if (threshold > -FLT_MAX) 
      cout <<" threshold "<<threshold;
    cout <<" : "<<argv[optind]<<" -> "<<argv[optind+1]<<endl;
  }

  ImageBase inimage,outimage;
  if (ImageIO::Read(argv[optind],inimage)<0) exit(1);

  if (inimage.GetStorageType() == ImageBase::ST_unsignedchar) {
    Image<unsigned char> img1(inimage),img2;
    switch (UseFilter){
    case median: //rankvaluefilter.Median(img1,img2,rad,threshold);
      BIASERR("Median filter disabled");
      break;
    case mean:
      //      meanfilterUC.Set(rad);
      meanfilterUC.Filter(img1,img2);
      break;
    default:
      cout <<"==> No filtertype specified."<<endl;
      usage();
      exit(1);
      break;
    }
    ImageIO::Write(argv[optind+1],img2);
  }

  if (inimage.GetStorageType() == ImageBase::ST_float) {
    Image<float> img1(inimage),img2;
    switch (UseFilter){
    case median: //rankvaluefilter.Median(img1,img2,rad,threshold);
       BIASERR("Median filter disabled");
     break;
    case mean:   
       //      meanfilterUC.Set(rad);
      meanfilterFLT.Filter(img1,img2);
      break;
    default:
      cout <<"==> No filtertype specified."<<endl;
      usage();
      exit(1);
      break;
    }
    ImageIO::Write(argv[optind+1],img2);
  }



  return 0;
}
